@@ -23,6 +23,7 @@
 #include <sys/stat.h>
 #include <math.h>
 
+#include <glib.h>
 #include <gdk_imlib.h>
 
 #include "config.h"
@@ -196,9 +197,9 @@ file_save_as_dialog_ok_callback (GtkWidget        *w,
     
     gtk_window_set_title (GTK_WINDOW (dialog), _("File already exists"));
     gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
-    snprintf(buffer, 300,
-	     _("The file '%s' already exists.\n"
-	     "Do you want to overwrite it?"), filename);
+    g_snprintf(buffer, 300,
+	       _("The file '%s' already exists.\n"
+		 "Do you want to overwrite it?"), filename);
     label = gtk_label_new (buffer);
   
     gtk_misc_set_padding (GTK_MISC (label), 10, 10);
@@ -340,242 +341,6 @@ file_import_from_xfig_callback (GtkWidget *widget, gpointer data)
   gtk_grab_add(window);
 }
 
-static void
-file_export_to_eps_dialog_ok_callback (GtkWidget        *w,
-				       GtkFileSelection *fs)
-{
-  char *filename;
-  Diagram *dia;
-  struct stat stat_struct;
-
-  dia = gtk_object_get_user_data(GTK_OBJECT(fs));
-
-  diagram_update_extents(dia);
-
-  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
-
-  if (stat(filename, &stat_struct)==0) {
-    GtkWidget *dialog = NULL;
-    GtkWidget *button, *label;
-    char buffer[300];
-    int result;
-
-    dialog = gtk_dialog_new();
-  
-    gtk_signal_connect (GTK_OBJECT (dialog), "destroy", 
-					 GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-    
-    gtk_window_set_title (GTK_WINDOW (dialog), _("File already exists"));
-    gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
-    snprintf(buffer, 300,
-	     _("The file '%s' already exists.\n"
-	     "Do you want to overwrite it?"), filename);
-    label = gtk_label_new (buffer);
-  
-    gtk_misc_set_padding (GTK_MISC (label), 10, 10);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			label, TRUE, TRUE, 0);
-  
-    gtk_widget_show (label);
-
-    result = FALSE;
-    
-    button = gtk_button_new_with_label (_("Yes"));
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
-			button, TRUE, TRUE, 0);
-    gtk_widget_grab_default (button);
-    gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC(set_true_callback),
-			&result);
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    gtk_widget_show (button);
-    
-    button = gtk_button_new_with_label (_("No"));
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
-			button, TRUE, TRUE, 0);
-    
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    
-    gtk_widget_show (button);
-
-    gtk_widget_show (dialog);
-
-    /* Make dialog modal: */
-    gtk_widget_grab_focus(dialog);
-    gtk_grab_add(dialog);
-
-    gtk_main();
-
-    if (result==FALSE) {
-      gtk_grab_remove(GTK_WIDGET(fs));
-      gtk_widget_destroy(GTK_WIDGET(fs));
-      return;
-    }
-  }
-  
-  diagram_export_to_eps(dia, filename);
-  
-  gtk_grab_remove(GTK_WIDGET(fs));
-  gtk_widget_destroy(GTK_WIDGET(fs));
-}
-
-void
-file_export_to_eps_callback(GtkWidget *widget, gpointer data)
-{
-  DDisplay *ddisp;
-  GtkWidget *window = NULL;
-  GtkFileSelection *fs;
-
-  ddisp = ddisplay_active();
-
-  window = gtk_file_selection_new (_("Export to postscript"));
-  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-
-  fs = GTK_FILE_SELECTION (window);
-
-  gtk_object_set_user_data(GTK_OBJECT(window), ddisp->diagram);
-
-  gtk_signal_connect (GTK_OBJECT (fs->ok_button), "clicked",
-		      GTK_SIGNAL_FUNC(file_export_to_eps_dialog_ok_callback),
-		      window);
-
-  gtk_signal_connect_object (GTK_OBJECT (fs->cancel_button),
-			     "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			     GTK_OBJECT (window));
-
-  /* put a .eps extention on the file name */
-  set_default_file_selection_extension (window, ".eps");
-  gtk_widget_show (window);
-
-  /* Make dialog modal: */
-  gtk_widget_grab_focus(window);
-  gtk_grab_add(window);
-}
-
-static void
-file_export_to_svg_dialog_ok_callback (GtkWidget        *w,
-				       GtkFileSelection *fs)
-{
-  char *filename;
-  Diagram *dia;
-  struct stat stat_struct;
-
-  dia = gtk_object_get_user_data(GTK_OBJECT(fs));
-
-  diagram_update_extents(dia);
-
-  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
-
-  if (stat(filename, &stat_struct)==0) {
-    GtkWidget *dialog = NULL;
-    GtkWidget *button, *label;
-    char buffer[300];
-    int result;
-
-    dialog = gtk_dialog_new();
-  
-    gtk_signal_connect (GTK_OBJECT (dialog), "destroy", 
-					 GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-    
-    gtk_window_set_title (GTK_WINDOW (dialog), _("File already exists"));
-    gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
-    snprintf(buffer, 300,
-	     _("The file '%s' already exists.\n"
-	     "Do you want to overwrite it?"), filename);
-    label = gtk_label_new (buffer);
-  
-    gtk_misc_set_padding (GTK_MISC (label), 10, 10);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			label, TRUE, TRUE, 0);
-  
-    gtk_widget_show (label);
-
-    result = FALSE;
-    
-    button = gtk_button_new_with_label (_("Yes"));
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
-			button, TRUE, TRUE, 0);
-    gtk_widget_grab_default (button);
-    gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC(set_true_callback),
-			&result);
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    gtk_widget_show (button);
-    
-    button = gtk_button_new_with_label (_("No"));
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
-			button, TRUE, TRUE, 0);
-    
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    
-    gtk_widget_show (button);
-
-    gtk_widget_show (dialog);
-
-    /* Make dialog modal: */
-    gtk_widget_grab_focus(dialog);
-    gtk_grab_add(dialog);
-
-    gtk_main();
-
-    if (result==FALSE) {
-      gtk_grab_remove(GTK_WIDGET(fs));
-      gtk_widget_destroy(GTK_WIDGET(fs));
-      return;
-    }
-  }
-  
-  diagram_export_to_svg(dia, filename);
-  
-  gtk_grab_remove(GTK_WIDGET(fs));
-  gtk_widget_destroy(GTK_WIDGET(fs));
-}
-
-void
-file_export_to_svg_callback(GtkWidget *widget, gpointer data)
-{
-  DDisplay *ddisp;
-  GtkWidget *window = NULL;
-  GtkFileSelection *fs;
-
-  ddisp = ddisplay_active();
-
-  window = gtk_file_selection_new (_("Export to SVG"));
-  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-
-  fs = GTK_FILE_SELECTION (window);
-
-  gtk_object_set_user_data(GTK_OBJECT(window), ddisp->diagram);
-
-  gtk_signal_connect (GTK_OBJECT (fs->ok_button), "clicked",
-		      GTK_SIGNAL_FUNC(file_export_to_svg_dialog_ok_callback),
-		      window);
-
-  gtk_signal_connect_object (GTK_OBJECT (fs->cancel_button),
-			     "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			     GTK_OBJECT (window));
-
-  /* put a .eps extention on the file name */
-  set_default_file_selection_extension (window, ".svg");
-  gtk_widget_show (window);
-
-  /* Make dialog modal: */
-  gtk_widget_grab_focus(window);
-  gtk_grab_add(window);
-}
-
 void
 file_pagesetup_callback(GtkWidget *widget, gpointer data)
 {
@@ -612,7 +377,7 @@ file_new_callback(GtkWidget *widget, gpointer data)
   static int untitled_nr = 1;
   char buffer[24];
 
-  snprintf(buffer, 24, _("Untitled-%d"), untitled_nr++);
+  g_snprintf(buffer, 24, _("Untitled-%d"), untitled_nr++);
   
   dia = new_diagram(buffer);
   ddisp = new_display(dia);
@@ -752,15 +517,6 @@ edit_redo_callback(GtkWidget *widget, gpointer data)
   diagram_flush(dia);
 } 
 
-void
-logo_expose_callback(GtkWidget *widget, GdkEventExpose *event)
-{
-  if (logo) {
-    gdk_imlib_paste_image(logo, widget->window, event->area.x, event->area.y, 
-			  event->area.width, event->area.height);
-  }
-}
-
 void
 help_about_callback(GtkWidget *widget, gpointer data)
 {
@@ -772,7 +528,7 @@ help_about_callback(GtkWidget *widget, gpointer data)
   GtkWidget *button;
   char str[100];
 
-  GtkWidget *drawarea;
+  GtkWidget *gpixmap;
 
   dialog = gtk_dialog_new ();
   gtk_window_set_wmclass (GTK_WINDOW (dialog), "about_dialog", "Dia");
@@ -787,8 +543,11 @@ help_about_callback(GtkWidget *widget, gpointer data)
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
   gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);
 
-  g_snprintf(str, sizeof(str), "%s/dia_logo.png", DATADIR);
-  logo = gdk_imlib_load_image(str);
+  if (!logo) {
+      g_snprintf(str, sizeof(str), "%s/dia_logo.png", DATADIR);
+      logo = gdk_imlib_load_image(str);
+      gdk_imlib_render(logo, logo->rgb_width, logo->rgb_height);
+  }
 
   if (logo) {
     frame = gtk_frame_new (NULL);
@@ -796,12 +555,9 @@ help_about_callback(GtkWidget *widget, gpointer data)
     gtk_container_set_border_width (GTK_CONTAINER (frame), 1);
     gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 1);
 
-    drawarea = gtk_drawing_area_new();
-    gtk_container_add (GTK_CONTAINER(frame), drawarea);
-    gtk_signal_connect (GTK_OBJECT (drawarea), "expose_event",
-			(GtkSignalFunc) logo_expose_callback, NULL);
-    
-    gtk_drawing_area_size(GTK_DRAWING_AREA(drawarea), logo->rgb_width, logo->rgb_height);
+    gpixmap = gtk_pixmap_new(logo->pixmap, logo->shape_mask);
+
+    gtk_container_add (GTK_CONTAINER(frame), gpixmap);
   }
 
   frame = gtk_frame_new (NULL);
@@ -813,7 +569,7 @@ help_about_callback(GtkWidget *widget, gpointer data)
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 1);
   gtk_container_add (GTK_CONTAINER (frame), vbox);
 
-  sprintf(str, _("Dia v %s by Alexander Larsson"), VERSION);
+  g_snprintf(str, sizeof(str), _("Dia v %s by Alexander Larsson"), VERSION);
   label = gtk_label_new (str);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 2);
 
