@@ -25,8 +25,6 @@ extern void file_open_callback(GtkWidget *widget, gpointer data);
 extern void file_save_callback(GtkWidget *widget, gpointer data);
 extern void file_save_as_callback(GtkWidget *widget, gpointer data);
 extern void file_import_from_xfig_callback(GtkWidget *widget, gpointer data);
-extern void file_export_to_eps_callback(GtkWidget *widget, gpointer data);
-extern void file_export_to_svg_callback(GtkWidget *widget, gpointer data);
 extern void file_pagesetup_callback(GtkWidget *widget, gpointer data);
 extern void file_print_callback(GtkWidget *widget, gpointer data);
 extern void file_close_callback(GtkWidget *widget, gpointer data);
