@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@ static GList *stored_list = NULL;
 static void free_stored(void)
 {
   if (stored_list != NULL) {
-    object_destroy_list(stored_list);
+    destroy_object_list(stored_list);
     stored_list = NULL;
   }
 }
