@@ -24,7 +24,6 @@
 #include "group.h"
 #include "object_ops.h"
 #include "render_eps.h"
-#include "render_svg.h"
 #include "focus.h"
 #include "message.h"
 #include "menus.h"
@@ -743,28 +742,6 @@ diagram_import_from_xfig(Diagram *dia, char *filename)
   printf("Fooled you!  Can't import %s yet!\n", filename);
 }
 
-void
-diagram_export_to_eps(Diagram *dia, char *filename)
-{
-  RendererEPS *renderer;
- 
-  renderer = new_eps_renderer(dia, filename);
-
-  data_render(dia->data, (Renderer *)renderer, NULL, NULL, NULL);
-  destroy_eps_renderer(renderer);
-}
-
-void
-diagram_export_to_svg(Diagram *dia, char *filename)
-{
-  RendererSVG *renderer;
-
-  renderer = new_svg_renderer(dia, filename);
-
-  data_render(dia->data, (Renderer *)renderer, NULL, NULL, NULL);
-  destroy_svg_renderer(renderer);
-}
-
 int diagram_modified_exists(void)
 {
   GList *list;
