@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,9 @@
 #include <stdio.h>
 #include <math.h>
 
+#include "config.h"
+#include "intl.h"
+
 #include "display.h"
 #include "group.h"
 #include "interface.h"
@@ -28,7 +31,7 @@
 #include "menus.h"
 #include "cut_n_paste.h"
 #include "message.h"
-
+#include "preferences.h"
 
 static GHashTable *display_ht = NULL;
 static GdkCursor *current_cursor = NULL;
@@ -42,6 +45,48 @@ typedef struct _IRectangle {
 
 static guint display_hash(DDisplay *ddisp);
 
+static void
+update_zoom_status(DDisplay *ddisp)
+{
+  GtkStatusbar *statusbar;
+  guint context_id;
+  gchar *zoom_text;
+
+  statusbar = GTK_STATUSBAR (ddisp->zoom_status);
+  context_id = gtk_statusbar_get_context_id (statusbar, "Zoom");
+  
+  gtk_statusbar_pop (statusbar, context_id); 
+  zoom_text = g_malloc (sizeof (guchar) * (strlen (_("Zoom")) + 1 + 8 + 1));
+  sprintf (zoom_text, "%s % 7.1f%c", _("Zoom"),
+	   ddisp->zoom_factor * 100.0 / DDISPLAY_NORMAL_ZOOM, '%');
+  gtk_statusbar_push (statusbar, context_id, zoom_text);
+
+  g_free (zoom_text);
+}
+
+static void
+update_modified_status(DDisplay *ddisp)
+{
+  GtkStatusbar *statusbar;
+  guint context_id;
+
+  if (ddisp->diagram->modified)
+  {
+    statusbar = GTK_STATUSBAR (ddisp->modified_status);
+    context_id = gtk_statusbar_get_context_id (statusbar, "Changed");
+  
+    gtk_statusbar_pop (statusbar, context_id); 
+    gtk_statusbar_push (statusbar, context_id, _("Diagram modified!"));
+  }
+  else
+  {
+    statusbar = GTK_STATUSBAR (ddisp->modified_status);
+    context_id = gtk_statusbar_get_context_id (statusbar, "Changed");
+  
+    gtk_statusbar_pop (statusbar, context_id); 
+  }
+}
+
 DDisplay *
 new_display(Diagram *dia)
 {
@@ -52,27 +97,21 @@ new_display(Diagram *dia)
 
   ddisp->diagram = dia;
 
-  ddisp->origo.x = 0.0;
-  ddisp->origo.y = 0.0;
-  ddisp->zoom_factor = DDISPLAY_NORMAL_ZOOM;
-
-  ddisp->grid.visible = TRUE;
-  ddisp->grid.snap = FALSE;
-  ddisp->grid.width_x = 1.0;
-  ddisp->grid.width_y = 1.0;
+  ddisp->grid.visible = prefs.grid.visible;
+  ddisp->grid.snap = prefs.grid.snap;
+  ddisp->grid.width_x = prefs.grid.x;
+  ddisp->grid.width_y = prefs.grid.y;
   ddisp->grid.gc = NULL;
   ddisp->grid.dialog = NULL;
   ddisp->grid.entry_x = NULL;
   ddisp->grid.entry_y = NULL;
-  
-  ddisp->pixmap = NULL;
-  ddisp->width = 0;
-  ddisp->height = 0;
-  ddisp->clip_region = NULL;
+
+  ddisp->show_cx_pts = prefs.show_cx_pts;
+
+  ddisp->autoscroll = TRUE;
+
   ddisp->renderer = NULL;
   
-  ddisp->bg_gc = NULL;
-
   ddisp->update_areas = NULL;
   ddisp->display_areas = NULL;
 
@@ -82,8 +121,24 @@ new_display(Diagram *dia)
   } else {
     filename++;
   }
-  
-  create_display_shell(ddisp, 256, 256, filename);
+
+  diagram_add_ddisplay(dia, ddisp);
+
+  ddisp->origo.x = 0.0;
+  ddisp->origo.y = 0.0;
+  ddisp->zoom_factor = prefs.new_view.zoom/100.0*DDISPLAY_NORMAL_ZOOM;
+  ddisp->visible.left = 0.0;
+  ddisp->visible.top = 0.0;
+  ddisp->visible.right = prefs.new_view.width/ddisp->zoom_factor;
+  ddisp->visible.bottom = prefs.new_view.height/ddisp->zoom_factor;
+
+  create_display_shell(ddisp, prefs.new_view.width, prefs.new_view.height,
+		       filename);
+
+  ddisplay_set_origo(ddisp, 0.0, 0.0);
+
+  ddisplay_update_statusbar (ddisp);
+  ddisplay_update_scrollbars(ddisp);
 
   if (!display_ht)
     display_ht = g_hash_table_new ((GHashFunc) display_hash, NULL);
@@ -108,9 +163,13 @@ ddisplay_transform_coords(DDisplay *ddisp,
 			  int *xi, int *yi)
 {
   Rectangle *visible = &ddisp->visible;
+  int width = ddisp->renderer->renderer.pixel_width;
+  int height = ddisp->renderer->renderer.pixel_height;
   
-  *xi = ROUND ( (x - visible->left)  * (real)ddisp->width / (visible->right - visible->left) );
-  *yi = ROUND ( (y - visible->top)  * (real)ddisp->height / (visible->bottom - visible->top) );
+  *xi = ROUND ( (x - visible->left)  * (real)width /
+		(visible->right - visible->left) );
+  *yi = ROUND ( (y - visible->top)  * (real)height /
+		(visible->bottom - visible->top) );
 }
 
 /* Takes real length and returns pixel length */
@@ -134,9 +193,11 @@ ddisplay_untransform_coords(DDisplay *ddisp,
 			    coord *x, coord *y)
 {
   Rectangle *visible = &ddisp->visible;
+  int width = ddisp->renderer->renderer.pixel_width;
+  int height = ddisp->renderer->renderer.pixel_height;
   
-  *x = visible->left + xi*(visible->right - visible->left) / (real)ddisp->width;
-  *y = visible->top +  yi*(visible->bottom - visible->top) / (real)ddisp->height;
+  *x = visible->left + xi*(visible->right - visible->left) / (real)width;
+  *y = visible->top +  yi*(visible->bottom - visible->top) / (real)height;
 }
 
 
@@ -170,6 +231,8 @@ ddisplay_add_update(DDisplay *ddisp, Rectangle *rect)
   Rectangle *r;
   int top,bottom,left,right;
   Rectangle *visible;
+  int width = ddisp->renderer->renderer.pixel_width;
+  int height = ddisp->renderer->renderer.pixel_height;
 
   if (!rectangle_intersects(rect, &ddisp->visible))
     return;
@@ -187,10 +250,14 @@ ddisplay_add_update(DDisplay *ddisp, Rectangle *rect)
   }
   
   visible = &ddisp->visible;
-  left = floor( (r->left - visible->left)  * (real)ddisp->width / (visible->right - visible->left) ) - 1;
-  top = floor( (r->top - visible->top)  * (real)ddisp->height / (visible->bottom - visible->top) ) - 1; 
-  right = ceil( (r->right - visible->left)  * (real)ddisp->width / (visible->right - visible->left) ) + 1;
-  bottom = ceil( (r->bottom - visible->top)  * (real)ddisp->height / (visible->bottom - visible->top) ) + 1;
+  left = floor( (r->left - visible->left)  * (real)width /
+		(visible->right - visible->left) ) - 1;
+  top = floor( (r->top - visible->top)  * (real)height /
+	       (visible->bottom - visible->top) ) - 1; 
+  right = ceil( (r->right - visible->left)  * (real)width /
+		(visible->right - visible->left) ) + 1;
+  bottom = ceil( (r->bottom - visible->top)  * (real)height /
+		 (visible->bottom - visible->top) ) + 1;
 
   ddisplay_add_display_area(ddisp,
 			    left, top, 
@@ -228,115 +295,113 @@ ddisplay_flush(DDisplay *ddisp)
 {
   GSList *l;
   IRectangle *ir;
-  Rectangle *r;
-  GdkRectangle clip_rect;
-  int x1,y1;
-  int x2,y2;
-
+  Rectangle *r, totrect;
+  Renderer *renderer;
+  
   /* Renders updates to pixmap + copies display_areas to canvas(screen) */
 
-  if (ddisp->clip_region != NULL)
-    gdk_region_destroy(ddisp->clip_region);
+  renderer = &ddisp->renderer->renderer;
 
-  ddisp->clip_region =  gdk_region_new();
+  (renderer->interactive_ops->clip_region_clear)(renderer);
 
   l = ddisp->update_areas;
+  if (l==NULL)
+    totrect = ddisp->visible;
+  else 
+    totrect = *(Rectangle *) l->data;
+
   while(l!=NULL) {
     r = (Rectangle *) l->data;
 
-    ddisplay_transform_coords(ddisp, r->left, r->top,
-			      &x1, &y1);
-    ddisplay_transform_coords(ddisp, r->right, r->bottom,
-			      &x2, &y2);
-    clip_rect.x = x1;
-    clip_rect.y = y1;
-    clip_rect.width = x2 - x1 + 1;
-    clip_rect.height = y2 - y1 + 1;
-    
-    ddisp->clip_region =
-      gdk_region_union_with_rect( ddisp->clip_region, &clip_rect );
+    rectangle_union(&totrect, r);
+    (renderer->interactive_ops->clip_region_add_rect)(renderer,  r);
     
     l = g_slist_next(l);
   }
-  /* Free list: */
+  
+  /* Free update_areas list: */
+  l = ddisp->update_areas;
+  while(l!=NULL) {
+    g_free(l->data);
+    l = g_slist_next(l);
+  }
   g_slist_free(ddisp->update_areas);
   ddisp->update_areas = NULL;
 
-  ddisplay_render_pixmap(ddisp);
+  totrect.left -= 0.1;
+  totrect.right += 0.1;
+  totrect.top -= 0.1;
+  totrect.bottom += 0.1;
   
+  ddisplay_render_pixmap(ddisp, &totrect);
 
   l = ddisp->display_areas;
   while(l!=NULL) {
     ir = (IRectangle *) l->data;
-    gdk_draw_pixmap(ddisp->canvas->window,
-		    ddisp->canvas->style->fg_gc[ GTK_WIDGET_STATE (ddisp->canvas)],
-		    ddisp->pixmap,
-		    ir->left, ir->top, 
-		    ir->left, ir->top, 
-		    ir->right - ir->left,
-		    ir->bottom - ir->top);
+
+    renderer_gdk_copy_to_window(ddisp->renderer, ddisp->canvas->window,
+				ir->left, ir->top,
+				ir->right - ir->left, ir->bottom - ir->top);
     
     l = g_slist_next(l);
   }
- 
+
+  /* Free display_areas list */
+  l = ddisp->display_areas;
+  while(l!=NULL) {
+    g_free(l->data);
+    l = g_slist_next(l);
+  }
   g_slist_free(ddisp->display_areas);
   ddisp->display_areas = NULL;
 }
 
+static void
+ddisplay_obj_render(Object *obj, Renderer *renderer,
+		    int active_layer,
+		    gpointer data)
+{
+  DDisplay *ddisp = (DDisplay *)data;
+  int i;
+
+  obj->ops->draw(obj, renderer);
+  if (active_layer && ddisp->show_cx_pts) {
+    for (i=0;i<obj->num_connections;i++) {
+      connectionpoint_draw(obj->connections[i], ddisp);
+    }
+  }
+}
+
 void
-ddisplay_render_pixmap(DDisplay *ddisp)
+ddisplay_render_pixmap(DDisplay *ddisp, Rectangle *update)
 {
   GList *list;
   Object *obj;
   int i;
+  Renderer *renderer;
   
-  if (ddisp->pixmap==NULL) {
-    printf("Pixmap was NULL!!\n");
+  if (ddisp->renderer==NULL) {
+    printf("ERROR! Renderer was NULL!!\n");
     return;
   }
 
-  if (ddisp->renderer==NULL) {
-    ddisp->renderer = new_gdk_renderer(ddisp);
-  }
-  
-  if(!ddisp->bg_gc) {
-    GdkColor col;
+  renderer = &ddisp->renderer->renderer;
 
-    ddisp->bg_gc = gdk_gc_new(ddisp->pixmap);
-
-    color_convert(&ddisp->diagram->bg_color, &col);
-    gdk_gc_set_foreground(ddisp->bg_gc, &col);
-  }
-    
-  gdk_gc_set_clip_region(ddisp->bg_gc, ddisp->clip_region);
   /* Erase background */
-  gdk_draw_rectangle (ddisp->pixmap,
-		      ddisp->bg_gc,
-		      TRUE,
-		      0, 0,
-		      ddisp->width,
-		      ddisp->height);
+  (renderer->interactive_ops->fill_pixel_rect)(renderer,
+					       0, 0,
+					       renderer->pixel_width,
+					       renderer->pixel_height,
+					       &ddisp->diagram->data->bg_color);
 
   /* Draw grid */
-  grid_draw(ddisp);
+  grid_draw(ddisp, update);
 
-
-  gdk_gc_set_clip_region(ddisp->renderer->render_gc, ddisp->clip_region);
-  /* Draw all objects: */
-  list = ddisp->diagram->objects;
-  while (list!=NULL) {
-    obj = (Object *) list->data;
-
-    obj->ops->draw(obj, (Renderer *)ddisp->renderer);
-    for (i=0;i<obj->num_connections;i++) {
-      connectionpoint_draw(obj->connections[i], ddisp);
-    }
-
-    list = g_list_next(list);
-  }
+  data_render(ddisp->diagram->data, (Renderer *)ddisp->renderer, update,
+	      ddisplay_obj_render, (gpointer) ddisp);
   
   /* Draw handles for all selected objects */
-  list = ddisp->diagram->selected;
+  list = ddisp->diagram->data->selected;
   while (list!=NULL) {
     obj = (Object *) list->data;
 
@@ -350,14 +415,18 @@ ddisplay_render_pixmap(DDisplay *ddisp)
 void
 ddisplay_update_scrollbars(DDisplay *ddisp)
 {
-  Rectangle *extents = &ddisp->diagram->extents;
+  Rectangle *extents = &ddisp->diagram->data->extents;
   Rectangle *visible = &ddisp->visible;
   GtkAdjustment *hsbdata, *vsbdata;
+  real extra_border_x, extra_border_y;
 
+  extra_border_x = (visible->right - visible->left);
+  extra_border_y = (visible->bottom - visible->top);
+  
   hsbdata = ddisp->hsbdata;
   /* Horizontal: */
-  hsbdata->lower = MIN(extents->left, visible->left);
-  hsbdata->upper = MAX(extents->right, visible->right);
+  hsbdata->lower = MIN(extents->left, visible->left)  - extra_border_x;
+  hsbdata->upper = MAX(extents->right, visible->right)  + extra_border_x;
   hsbdata->page_size = visible->right - visible->left - 0.0001;
   /* remove some to fix strange behaviour in gtk_range_adjustment_changed */
   hsbdata->page_increment = (visible->right - visible->left) / 2.0;
@@ -368,8 +437,8 @@ ddisplay_update_scrollbars(DDisplay *ddisp)
 
   /* Vertical: */
   vsbdata = ddisp->vsbdata;
-  vsbdata->lower = MIN(extents->top, visible->top);
-  vsbdata->upper = MAX(extents->bottom, visible->bottom);
+  vsbdata->lower = MIN(extents->top, visible->top)  - extra_border_y;
+  vsbdata->upper = MAX(extents->bottom, visible->bottom) + extra_border_y;
   vsbdata->page_size = visible->bottom - visible->top - 0.00001;
   /* remove some to fix strange behaviour in gtk_range_adjustment_changed */
   vsbdata->page_increment = (visible->bottom - visible->top) / 2.0;
@@ -382,8 +451,10 @@ ddisplay_update_scrollbars(DDisplay *ddisp)
 void
 ddisplay_set_origo(DDisplay *ddisp, coord x, coord y)
 {
-  Rectangle *extents = &ddisp->diagram->extents;
+  Rectangle *extents = &ddisp->diagram->data->extents;
   Rectangle *visible = &ddisp->visible;
+  int width = ddisp->renderer->renderer.pixel_width;
+  int height = ddisp->renderer->renderer.pixel_height;
     
   /*  updaterar origo+visible+rulers */
   ddisp->origo.x = x;
@@ -397,8 +468,8 @@ ddisplay_set_origo(DDisplay *ddisp, coord x, coord y)
 
   visible->left = ddisp->origo.x;
   visible->top = ddisp->origo.y;
-  visible->right = ddisp->origo.x + ddisp->width/ddisp->zoom_factor;
-  visible->bottom = ddisp->origo.y + ddisp->height/ddisp->zoom_factor;
+  visible->right = ddisp->origo.x + width/ddisp->zoom_factor;
+  visible->bottom = ddisp->origo.y + height/ddisp->zoom_factor;
 
   gtk_ruler_set_range  (GTK_RULER (ddisp->hrule),
 			visible->left,
@@ -435,6 +506,56 @@ ddisplay_zoom(DDisplay *ddisp, Point *point, real magnify)
   ddisplay_update_scrollbars(ddisp);
   ddisplay_add_update_all(ddisp);
   ddisplay_flush(ddisp);
+
+  update_zoom_status (ddisp);
+}
+
+gboolean
+ddisplay_autoscroll(DDisplay *ddisp, int x, int y)
+{
+  guint16 width, height;
+  Point scroll;
+  
+  if (! ddisp->autoscroll)
+    return FALSE;
+
+  scroll.x = scroll.y = 0;
+
+  width = GTK_WIDGET(ddisp->canvas)->allocation.width;
+  height = GTK_WIDGET(ddisp->canvas)->allocation.height;
+
+  if (x < 0)
+  {
+    scroll.x = x;
+  }
+  else if ( x > width)
+  {
+    scroll.x = x - width;
+  }
+
+  if (y < 0)
+  {
+    scroll.y = y;
+  }
+  else if (y > height)
+  {
+    scroll.y = y - height;
+  }
+
+  if ((scroll.x != 0) || (scroll.y != 0))
+  {
+    scroll.x = ddisplay_untransform_length(ddisp, scroll.x);
+    scroll.y = ddisplay_untransform_length(ddisp, scroll.y);
+
+    ddisplay_scroll(ddisp, &scroll);
+    ddisplay_flush(ddisp);
+
+    return TRUE;
+  }
+  else
+  {
+    return FALSE;
+  }
 }
 
 void ddisplay_scroll(DDisplay *ddisp, Point *delta)
@@ -450,7 +571,7 @@ void ddisplay_scroll(DDisplay *ddisp, Point *delta)
   width = visible->right - visible->left;
   height = visible->bottom - visible->top;
   
-  extents = ddisp->diagram->extents;
+  extents = ddisp->diagram->data->extents;
   rectangle_union(&extents, visible);
   
   if (new_origo.x < extents.left)
@@ -514,15 +635,11 @@ void
 ddisplay_resize_canvas(DDisplay *ddisp,
 		       int width,  int height)
 {
-  if (ddisp->pixmap != NULL) {
-    gdk_pixmap_unref(ddisp->pixmap);
+  if (ddisp->renderer==NULL) {
+    ddisp->renderer = new_gdk_renderer(ddisp);
   }
   
-  ddisp->pixmap =
-    gdk_pixmap_new(ddisp->canvas->window,
-		   width, height, -1);
-  ddisp->width = width;
-  ddisp->height = height;
+  gdk_renderer_set_size(ddisp->renderer, ddisp->canvas->window, width, height);
 
   ddisplay_set_origo(ddisp, ddisp->origo.x, ddisp->origo.y);
 
@@ -533,18 +650,25 @@ ddisplay_resize_canvas(DDisplay *ddisp,
 extern DDisplay *ddisplay_active(void)
 {
   DDisplay *ddisp;
-  GtkWidget *event_widget;
-  GtkWidget *toplevel_widget;
+  GtkWidget *event_widget = NULL;
+  GtkWidget *toplevel_widget = NULL;
   GdkEvent *event;
 
-  /*  If the popup shell is valid, then get the gdisplay associated with that shell  */
+  /*  If the popup shell is valid, then get the gdisplay
+      associated with that shell  */
   event = gtk_get_current_event ();
-  event_widget = gtk_get_event_widget (event);
-  gdk_event_free (event);
+  if (event)
+    {
+      event_widget = gtk_get_event_widget (event);
+      gdk_event_free (event);
+    }
 
   if (event_widget == NULL)
     return NULL;
 
+  if (display_ht == NULL)
+    return NULL;
+
   toplevel_widget = gtk_widget_get_toplevel (event_widget);
   ddisp = g_hash_table_lookup (display_ht, toplevel_widget);
 
@@ -556,8 +680,10 @@ extern DDisplay *ddisplay_active(void)
     return ddisp;
   }
 
-  message_error("Internal error: "
-		"Strange, shouldn't come here (ddisplay_active())"); 
+  /*
+  message_error(_("Internal error: "
+		"Strange, shouldn't come here (ddisplay_active())")); 
+  */
   return NULL;
 }
 
@@ -602,11 +728,11 @@ ddisplay_close(DDisplay *ddisp)
   
   dialog = gtk_dialog_new();
 
-  gtk_window_set_title (GTK_WINDOW (dialog), "Really close?");
-  gtk_container_border_width (GTK_CONTAINER (dialog), 0);
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Really close?"));
+  gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
   
-  label = gtk_label_new ("This diagram has not been saved.\n"
-			 "Are you sure you want to close this window?");
+  label = gtk_label_new (_("This diagram has not been saved.\n"
+			 "Are you sure you want to close this window?"));
   
   gtk_misc_set_padding (GTK_MISC (label), 10, 10);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
@@ -614,7 +740,7 @@ ddisplay_close(DDisplay *ddisp)
   
   gtk_widget_show (label);
   
-  button = gtk_button_new_with_label ("Yes");
+  button = gtk_button_new_with_label (_("Yes"));
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
 		      button, TRUE, TRUE, 0);
@@ -625,7 +751,7 @@ ddisplay_close(DDisplay *ddisp)
 		      dialog);
   gtk_widget_show (button);
   
-  button = gtk_button_new_with_label ("No");
+  button = gtk_button_new_with_label (_("No"));
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
 		      button, TRUE, TRUE, 0);
@@ -640,37 +766,49 @@ ddisplay_close(DDisplay *ddisp)
   gtk_widget_grab_focus(dialog);
   gtk_grab_add(dialog);
   
-  gtk_widget_show (dialog);
+  gtk_widget_show(dialog);
 }
 
 void
-display_set_menu_sensitivity(DDisplay *ddisp)
+display_update_menu_state(DDisplay *ddisp)
 {
   Diagram *dia;
+  static int initialized = 0;
+
+  static GtkWidget *rulers;
+  static GtkWidget *visible_grid;
+  static GtkWidget *snap_to_grid;
+  static GtkWidget *show_cx_pts;
+  static GString *path;
+  char *display = "<Display>";
+
+  if (initialized==0) {
+    path = g_string_new (display);
+    g_string_append (path,_("/View/Show Rulers"));
+    rulers       = menus_get_item_from_path(path->str);
+    g_string_append (g_string_assign(path, display),_("/View/Visible Grid"));
+    visible_grid = menus_get_item_from_path(path->str);
+    g_string_append (g_string_assign(path, display),_("/View/Snap To Grid"));
+    snap_to_grid = menus_get_item_from_path(path->str);
+    g_string_append (g_string_assign(path, display),_("/View/Show Connection Points"));
+    show_cx_pts  = menus_get_item_from_path(path->str);
+
+    g_string_free (path,FALSE);
+    initialized = 1;
+  }
   
   dia = ddisp->diagram;
 
-  menus_set_sensitive("<Display>/Edit/Copy",
-		      dia->selected_count > 0);
-  menus_set_sensitive("<Display>/Edit/Cut",
-		      dia->selected_count > 0);
-  menus_set_sensitive("<Display>/Edit/Paste",
-		      cnp_exist_stored_objects());
-  menus_set_sensitive("<Display>/Edit/Delete",
-		      dia->selected_count > 0);
+  diagram_update_menu_sensitivity(dia);
 
-  menus_set_sensitive("<Display>/Objects/Place Under",
-		      dia->selected_count > 0);
-  menus_set_sensitive("<Display>/Objects/Place Over",
-		      dia->selected_count > 0);
-  menus_set_sensitive("<Display>/Objects/Group", dia->selected_count > 1);
-  menus_set_sensitive("<Display>/Objects/Ungroup",
-		      (dia->selected_count == 1) &&
-		      IS_GROUP((Object *)dia->selected->data));
-
-  menus_set_state ("<Display>/View/Toggle Rulers", GTK_WIDGET_VISIBLE (ddisp->hrule) ? 1 : 0);
-  menus_set_state ("<Display>/View/Visible Grid", ddisp->grid.visible);
-  menus_set_state ("<Display>/View/Snap To Grid", ddisp->grid.snap);
+  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(rulers),
+				 GTK_WIDGET_VISIBLE (ddisp->hrule) ? 1 : 0); 
+  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(visible_grid),
+				 ddisp->grid.visible);
+  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(snap_to_grid),
+				 ddisp->grid.snap);
+  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(show_cx_pts),
+				 ddisp->show_cx_pts); 
 }
 
 /* This is called when ddisp->shell is destroyed... */
@@ -678,25 +816,35 @@ void
 ddisplay_really_destroy(DDisplay *ddisp)
 {
   Diagram *dia;
-  
+  GSList *l;
+
   dia = ddisp->diagram;
   
   diagram_remove_ddisplay(dia, ddisp);
 
   grid_destroy_dialog(&ddisp->grid);
   
-  if (ddisp->pixmap != NULL)
-    gdk_pixmap_unref(ddisp->pixmap);
-
-  if (ddisp->bg_gc != NULL)
-    gdk_gc_unref(ddisp->bg_gc);
-
-  if (ddisp->clip_region != NULL)
-    gdk_region_destroy(ddisp->clip_region);
-      
+  destroy_gdk_renderer(ddisp->renderer);
+  
   g_hash_table_remove(display_ht, ddisp->shell);
   g_hash_table_remove(display_ht, ddisp->canvas);
 
+  /* Free update_areas list: */
+  l = ddisp->update_areas;
+  while(l!=NULL) {
+    g_free(l->data);
+    l = g_slist_next(l);
+  }
+  g_slist_free(ddisp->update_areas);
+  /* Free display_areas list */
+  l = ddisp->display_areas;
+  while(l!=NULL) {
+    g_free(l->data);
+    l = g_slist_next(l);
+  }
+  g_slist_free(ddisp->display_areas);
+
+  
   g_free(ddisp);
 }
 
@@ -740,3 +888,9 @@ ddisplay_set_cursor(DDisplay *ddisp, GdkCursor *cursor)
   gdk_window_set_cursor(ddisp->canvas->window, cursor);
 }
 
+void 
+ddisplay_update_statusbar(DDisplay *ddisp)
+{
+  update_zoom_status (ddisp);
+  update_modified_status (ddisp);
+}
