@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,21 +15,26 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-/* $Header: /lysator/cvsroot/dia/app/interface.c,v 1.24 1998/08/29 14:20:21 alla Exp $ */
+/* $Header: /cvs/gnome/dia/app/interface.c,v 1.23 1999/12/05 10:31:15 jamesh Exp $ */
 
+#include "config.h"
+#ifdef GNOME
+#include <gnome.h>
+#endif
+#include "intl.h"
 #include "interface.h"
 #include "menus.h"
 #include "disp_callbacks.h"
 #include "tool.h"
 #include "sheet.h"
 #include "app_procs.h"
+#include "arrows.h"
 #include "color_area.h"
 #include "linewidth_area.h"
+#include "lineprops_area.h"
 
 #include "pixmaps.h"
 
-#include "config.h"
-
 typedef struct _ToolButton ToolButton;
 
 typedef struct _ToolButtonData ToolButtonData;
@@ -51,40 +56,52 @@ struct _ToolButton
 static ToolButton tool_data[] =
 {
   { (char **) arrow_xpm,
-    "Modify object(s)",
+    N_("Modify object(s)"),
     { MODIFY_TOOL, NULL, NULL}
   },
   { (char **) magnify_xpm,
-    "Magnify",
+    N_("Magnify"),
     { MAGNIFY_TOOL, NULL, NULL}
   },
   { (char **) scroll_xpm,
-    "Scroll around the diagram",
+    N_("Scroll around the diagram"),
     { SCROLL_TOOL, NULL, NULL}
   },
   { NULL,
-    "Create Text",
+    N_("Create Text"),
     { CREATE_OBJECT_TOOL, "Standard - Text", NULL }
   },
   { NULL,
-    "Create Box",
+    N_("Create Box"),
     { CREATE_OBJECT_TOOL, "Standard - Box", NULL }
   },
   { NULL,
-    "Create Ellipse",
+    N_("Create Ellipse"),
     { CREATE_OBJECT_TOOL, "Standard - Ellipse", NULL }
   },
   { NULL,
-    "Create Line",
+    N_("Create Line"),
     { CREATE_OBJECT_TOOL, "Standard - Line", NULL }
   },
   { NULL,
-    "Create Arc",
+    N_("Create Arc"),
     { CREATE_OBJECT_TOOL, "Standard - Arc", NULL }
   },
   { NULL,
-    "Create Zigzagline",
+    N_("Create Zigzagline"),
     { CREATE_OBJECT_TOOL, "Standard - ZigZagLine", NULL }
+  },
+  { NULL,
+    N_("Create Polyline"),
+    { CREATE_OBJECT_TOOL, "Standard - PolyLine", NULL }
+  },
+  { NULL,
+    N_("Create Bezierline"),
+    { CREATE_OBJECT_TOOL, "Standard - BezierLine", NULL }
+  },
+  { NULL,
+    N_("Create Image"),
+    { CREATE_OBJECT_TOOL, "Standard - Image", NULL }
   }
 };
 
@@ -111,6 +128,7 @@ create_display_shell(DDisplay *ddisp,
 		     char *title)
 {
   GtkWidget *table;
+  GtkWidget *status_hbox;
   int s_width, s_height;
 
   s_width = gdk_screen_width ();
@@ -126,12 +144,16 @@ create_display_shell(DDisplay *ddisp,
 
   /*  The toplevel shell */
   ddisp->shell = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_widget_ref  (ddisp->shell);
   gtk_window_set_title (GTK_WINDOW (ddisp->shell), title);
-  gtk_window_set_wmclass (GTK_WINDOW (ddisp->shell), "diagram_window", "Diagram-program");
+  gtk_window_set_wmclass (GTK_WINDOW (ddisp->shell), "diagram_window",
+			  "Dia");
   gtk_window_set_policy (GTK_WINDOW (ddisp->shell), TRUE, TRUE, TRUE);
   gtk_object_set_user_data (GTK_OBJECT (ddisp->shell), (gpointer) ddisp);
-  gtk_widget_set_events (ddisp->shell, GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK);
+  gtk_widget_set_events (ddisp->shell,
+			 GDK_POINTER_MOTION_MASK |
+			 GDK_POINTER_MOTION_HINT_MASK |
+			 GDK_FOCUS_CHANGE_MASK);
+                      /* GDK_ALL_EVENTS_MASK */
 
   gtk_signal_connect (GTK_OBJECT (ddisp->shell), "delete_event",
                       GTK_SIGNAL_FUNC (ddisplay_delete),
@@ -141,12 +163,12 @@ create_display_shell(DDisplay *ddisp,
                       ddisp);
   
   /*  the table containing all widgets  */
-  table = gtk_table_new (3, 3, FALSE);
+  table = gtk_table_new (4, 3, FALSE);
   gtk_table_set_col_spacing (GTK_TABLE (table), 0, 1);
   gtk_table_set_col_spacing (GTK_TABLE (table), 1, 2);
   gtk_table_set_row_spacing (GTK_TABLE (table), 0, 1);
   gtk_table_set_row_spacing (GTK_TABLE (table), 1, 2);
-  gtk_container_border_width (GTK_CONTAINER (table), 2);
+  gtk_container_set_border_width (GTK_CONTAINER (table), 2);
   gtk_container_add (GTK_CONTAINER (ddisp->shell), table);
 
   /*  scrollbars, rulers, canvas, menu popup button  */
@@ -201,16 +223,32 @@ create_display_shell(DDisplay *ddisp,
                     GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
   /*  the popup menu  */
-  menus_get_image_menu (&ddisp->popup, &ddisp->accel);
+#ifdef GNOME
+  ddisp->popup = gnome_display_menus_create ();
+  ddisp->accel_group = gnome_popup_menu_get_accel_group(GTK_MENU(ddisp->popup));
+#else
+  menus_get_image_menu (&ddisp->popup, &ddisp->accel_group);
+#endif
 
   /*  the accelerator table/group for the popup */
-#ifdef GTK_HAVE_FEATURES_1_1_0
-  gtk_window_add_accel_group (GTK_WINDOW(ddisp->shell),
-				    ddisp->accel);
-#else
-  gtk_window_add_accelerator_table (GTK_WINDOW(ddisp->shell),
-				    ddisp->accel);
-#endif /* GTK_HAVE_FEATURES_1_1_0 */
+  gtk_window_add_accel_group (GTK_WINDOW(ddisp->shell), ddisp->accel_group);
+
+
+  /* the statusbars */
+  status_hbox = gtk_hbox_new (FALSE, 2);
+
+  ddisp->zoom_status = gtk_statusbar_new ();
+  ddisp->modified_status = gtk_statusbar_new ();
+  
+  gtk_box_pack_start (GTK_BOX (status_hbox), ddisp->zoom_status,
+		      FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (status_hbox), ddisp->modified_status,
+		      TRUE, TRUE, 
+		      0);
+
+  gtk_table_attach (GTK_TABLE (table), status_hbox, 0, 3, 3, 4,
+                    GTK_FILL, GTK_FILL, 0, 0);
+
 
   gtk_widget_show (ddisp->hsb);
   gtk_widget_show (ddisp->vsb);
@@ -218,6 +256,9 @@ create_display_shell(DDisplay *ddisp,
   gtk_widget_show (ddisp->hrule);
   gtk_widget_show (ddisp->vrule);
   gtk_widget_show (ddisp->canvas);
+  gtk_widget_show (ddisp->zoom_status);
+  gtk_widget_show (ddisp->modified_status);
+  gtk_widget_show (status_hbox);
   gtk_widget_show (table);
   gtk_widget_show (ddisp->shell);
 
@@ -240,12 +281,14 @@ tool_button_press (GtkWidget      *w,
 		    GdkEventButton *event,
 		    gpointer        data)
 {
-  /*
+  ToolButtonData *tooldata = (ToolButtonData *) data;
+
   if ((event->type == GDK_2BUTTON_PRESS) &&
-      (event->button == 1))
-    tools_options_dialog_show ();
-  */
-  
+      (event->button == 1)) {
+    tool_options_dialog_show (tooldata->type, tooldata->extra_data, tooldata->user_data);
+    return TRUE;
+  }
+
   return FALSE;
 }
 
@@ -267,7 +310,7 @@ create_tools(GtkWidget *parent)
 
   for (i = 0; i < NUM_TOOLS; i++) {
     tool_widgets[i] = button = gtk_radio_button_new (tool_group);
-    gtk_container_border_width (GTK_CONTAINER (button), 0);
+    gtk_container_set_border_width (GTK_CONTAINER (button), 0);
     tool_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
     gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
 
@@ -286,7 +329,10 @@ create_tools(GtkWidget *parent)
       ObjectType *type;
       type =
 	object_get_type((char *)tool_data[i].callback_data.extra_data);
-      pixmap_data = type->pixmap;
+      if (type == NULL)
+	pixmap_data = tool_data[0].icon_data;
+      else
+	pixmap_data = type->pixmap;
     } else {
       pixmap_data = tool_data[i].icon_data;
     }
@@ -311,7 +357,7 @@ create_tools(GtkWidget *parent)
 			&tool_data[i].callback_data);
     
     gtk_tooltips_set_tip (tool_tips, button,
-			  tool_data[i].tool_desc, NULL);
+			  gettext(tool_data[i].tool_desc), NULL);
     
     gtk_widget_show (pixmapwidget);
     gtk_widget_show (button);
@@ -354,14 +400,28 @@ create_sheet_page(GtkWidget *parent, Sheet *sheet)
   while (list != NULL) {
     sheet_obj = (SheetObject *) list->data;
     
-    pixmap = gdk_pixmap_create_from_xpm_d(parent->window, &mask, 
-					  &style->bg[GTK_STATE_NORMAL], 
-					  sheet_obj->pixmap);
+    
+    if (sheet_obj->pixmap != NULL) {
+      pixmap = gdk_pixmap_create_from_xpm_d(parent->window, &mask, 
+					    &style->bg[GTK_STATE_NORMAL], 
+					    sheet_obj->pixmap);
+    } else if (sheet_obj->pixmap_file != NULL) {
+      pixmap = gdk_pixmap_create_from_xpm(parent->window, &mask,
+					  &style->bg[GTK_STATE_NORMAL],
+					  sheet_obj->pixmap_file);
+    } else {
+      ObjectType *type;
+      type = object_get_type(sheet_obj->object_type);
+      pixmap = gdk_pixmap_create_from_xpm_d(parent->window, &mask, 
+					    &style->bg[GTK_STATE_NORMAL], 
+					    type->pixmap);
+    }
+
     pixmapwidget = gtk_pixmap_new(pixmap, mask);
 
     
     button = gtk_radio_button_new (tool_group);
-    gtk_container_border_width (GTK_CONTAINER (button), 0);
+    gtk_container_set_border_width (GTK_CONTAINER (button), 0);
     tool_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
     
     gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
@@ -381,7 +441,7 @@ create_sheet_page(GtkWidget *parent, Sheet *sheet)
 
     data = g_new(ToolButtonData, 1);
     data->type = CREATE_OBJECT_TOOL;
-    data->extra_data = sheet_obj->type->name;
+    data->extra_data = sheet_obj->object_type;
     data->user_data = sheet_obj->user_data;
     
     gtk_signal_connect (GTK_OBJECT (button), "toggled",
@@ -393,14 +453,14 @@ create_sheet_page(GtkWidget *parent, Sheet *sheet)
 			data);
 
     gtk_tooltips_set_tip (tool_tips, button,
-			  sheet_obj->description, NULL);
+			  gettext(sheet_obj->description), NULL);
 
     list = g_slist_next(list);
     i++;
   }
 
   gtk_widget_show(table);
-  gtk_container_add(GTK_CONTAINER(scrolled_window), table);
+  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), table);
   
   return scrolled_window;
 }
@@ -414,6 +474,7 @@ create_sheets(GtkWidget *parent)
   Sheet *sheet;
   GtkWidget *child;
   GtkWidget *label;
+  GtkWidget *menu_label;
 
   
   separator = gtk_hseparator_new ();
@@ -428,21 +489,26 @@ create_sheets(GtkWidget *parent)
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
   gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), TRUE);
   gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
-  gtk_container_border_width (GTK_CONTAINER (notebook), 1);
+  gtk_notebook_popup_enable (GTK_NOTEBOOK (notebook));
+  gtk_container_set_border_width (GTK_CONTAINER (notebook), 1);
   gtk_box_pack_start (GTK_BOX (parent), notebook, TRUE, TRUE, 0);
   
   list = get_sheets_list();
   while (list != NULL) {
     sheet = (Sheet *) list->data;
 
-    label = gtk_label_new( sheet->name );
+    label = gtk_label_new(gettext(sheet->name));
+    menu_label = gtk_label_new(gettext(sheet->name));
+    gtk_misc_set_alignment(GTK_MISC(menu_label), 0.0, 0.5);
     
     child = create_sheet_page(notebook, sheet);
     
-    gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
-				   child, label);
     gtk_widget_show(label);
+    gtk_widget_show(menu_label);
     gtk_widget_show_all(child);
+
+    gtk_notebook_append_page_menu (GTK_NOTEBOOK (notebook),
+				   child, label, menu_label);
     
     list = g_slist_next(list);
   }
@@ -480,12 +546,12 @@ create_color_area (GtkWidget *parent)
   gtk_widget_realize (frame);
 
   hbox = gtk_hbox_new (FALSE, 1);
-  gtk_container_border_width (GTK_CONTAINER (hbox), 0);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
   gtk_container_add (GTK_CONTAINER (frame), hbox);
   
   /* Color area: */
   alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-  gtk_container_border_width (GTK_CONTAINER (alignment), 3);
+  gtk_container_set_border_width (GTK_CONTAINER (alignment), 3);
   
   col_area = color_area_create (54, 42, default_pixmap, swap_pixmap);
   gtk_container_add (GTK_CONTAINER (alignment), col_area);
@@ -493,16 +559,16 @@ create_color_area (GtkWidget *parent)
 
   gtk_box_pack_start (GTK_BOX (hbox), alignment, TRUE, TRUE, 0);
 
-  gtk_tooltips_set_tip (tool_tips, col_area, "Foreground & background colors.  The small black "
+  gtk_tooltips_set_tip (tool_tips, col_area, _("Foreground & background colors.  The small black "
                         "and white squares reset colors.  The small arrows swap colors.  Double "
-                        "click to change colors.",
+                        "click to change colors."),
                         NULL);
 
   gtk_widget_show (alignment);
   
   /* Linewidth area: */
   alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-  gtk_container_border_width (GTK_CONTAINER (alignment), 3);
+  gtk_container_set_border_width (GTK_CONTAINER (alignment), 3);
   
   line_area = linewidth_area_create ();
   gtk_container_add (GTK_CONTAINER (alignment), line_area);
@@ -516,6 +582,44 @@ create_color_area (GtkWidget *parent)
   gtk_widget_show (frame);
 }
 
+static void
+change_start_arrow_style(Arrow arrow, gpointer user_data)
+{
+  attributes_set_default_start_arrow(arrow);
+}
+static void
+change_end_arrow_style(Arrow arrow, gpointer user_data)
+{
+  attributes_set_default_end_arrow(arrow);
+}
+static void
+change_line_style(LineStyle lstyle, real dash_length, gpointer user_data)
+{
+  attributes_set_default_line_style(lstyle, dash_length);
+}
+
+static void
+create_lineprops_area(GtkWidget *parent)
+{
+  GtkWidget *hbox;
+  GtkWidget *chooser;
+
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (parent), hbox, FALSE, FALSE, 0);
+  gtk_widget_show(hbox);
+
+  chooser = dia_arrow_chooser_new(TRUE, change_start_arrow_style, NULL);
+  gtk_box_pack_start(GTK_BOX(hbox), chooser, FALSE, TRUE, 0);
+  gtk_widget_show(chooser);
+
+  chooser = dia_line_chooser_new(change_line_style, NULL);
+  gtk_box_pack_start(GTK_BOX(hbox), chooser, TRUE, TRUE, 0);
+  gtk_widget_show(chooser);
+
+  chooser = dia_arrow_chooser_new(FALSE, change_end_arrow_style, NULL);
+  gtk_box_pack_start(GTK_BOX(hbox), chooser, FALSE, TRUE, 0);
+  gtk_widget_show(chooser);
+}
 
 void
 toolbox_delete (GtkWidget *widget, gpointer data)
@@ -536,12 +640,17 @@ create_toolbox ()
   GtkWidget *main_vbox;
   GtkWidget *vbox;
   GtkWidget *menubar;
-  MY_GTK_ACCEL_TYPE *accel;
+  GtkAccelGroup *accel_group;
+
+#ifdef GNOME
+  window = gnome_app_new ("Dia", _("Diagram Editor"));
+#else
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_widget_ref (window);
   gtk_window_set_title (GTK_WINDOW (window), "Dia v" VERSION);
   gtk_window_set_wmclass (GTK_WINDOW (window), "toolbox_window",
-			  "Diagram-program");
+			  "Dia");
+#endif
 
   gtk_signal_connect (GTK_OBJECT (window), "delete_event",
 		      GTK_SIGNAL_FUNC (toolbox_delete),
@@ -552,13 +661,22 @@ create_toolbox ()
 		      NULL);
 
   main_vbox = gtk_vbox_new (FALSE, 1);
-  gtk_container_border_width (GTK_CONTAINER (main_vbox), 1);
+  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 1);
+#ifdef GNOME
+  gnome_app_set_contents(GNOME_APP(window), main_vbox);
+#else
   gtk_container_add (GTK_CONTAINER (window), main_vbox);
+#endif
   gtk_widget_show (main_vbox);
 
-  menus_get_toolbox_menubar(&menubar, &accel);
+#ifdef GNOME
+  gnome_toolbox_menus_create(window);
+#else
+  menus_get_toolbox_menubar(&menubar, &accel_group);
+  gtk_accel_group_attach (accel_group, GTK_OBJECT (window));
   gtk_box_pack_start (GTK_BOX (main_vbox), menubar, FALSE, TRUE, 0);
   gtk_widget_show (menubar);
+#endif
   
   /*  tooltips  */
   tool_tips = gtk_tooltips_new ();
@@ -569,12 +687,13 @@ create_toolbox ()
   
   vbox = gtk_vbox_new (FALSE, 1);
   gtk_box_pack_start (GTK_BOX (main_vbox), vbox, TRUE, TRUE, 0);
-  gtk_container_border_width (GTK_CONTAINER (vbox), 0);
+  gtk_container_set_border_width (GTK_CONTAINER (vbox), 0);
   gtk_widget_show (vbox);
 
   create_tools (vbox);
   create_sheets (vbox);
   create_color_area (vbox);
+  create_lineprops_area (vbox);
   
   gtk_widget_show (window);
   toolbox_shell = window;
