@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,21 +20,22 @@
 
 #include <gtk/gtk.h>
 
-#ifdef GTK_HAVE_FEATURES_1_1_0
-#define MY_GTK_ACCEL_TYPE GtkAccelGroup
-#else
-#define MY_GTK_ACCEL_TYPE GtkAcceleratorTable
-#endif /* GTK_HAVE_FEATURES_1_1_0 */
+#include "config.h"
+#ifdef GNOME
+#include <gnome.h>
+#endif
 
-extern void menus_get_toolbox_menubar (GtkWidget         **menubar,
-				       MY_GTK_ACCEL_TYPE **accel);
-extern void menus_get_image_menu (GtkWidget         **menu,
-				  MY_GTK_ACCEL_TYPE **accel);
+#ifdef GNOME
+extern void gnome_toolbox_menus_create(GtkWidget* app);
+extern GtkWidget * gnome_display_menus_create(void);
+#endif
 
-extern void menus_set_sensitive       (char                 *path,
-				       int                   sensitive);
+extern void menus_get_toolbox_menubar(GtkWidget **menubar,
+				      GtkAccelGroup **accel);
+extern void menus_get_image_menu(GtkWidget **menu,
+				 GtkAccelGroup **accel);
 
-extern void menus_set_state (char *path, int   state);
+extern GtkWidget *menus_get_item_from_path(char *path);
 
 #endif /* MENUS_H */
 
