@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,8 @@ typedef struct _ModifyTool ModifyTool;
 enum ModifyToolState {
   STATE_NONE,
   STATE_MOVE_OBJECT,
-  STATE_MOVE_HANDLE
+  STATE_MOVE_HANDLE,
+  STATE_BOX_SELECT
 };
 
 struct _ModifyTool {
@@ -38,6 +39,16 @@ struct _ModifyTool {
   Object *object;
   Handle *handle;
   Point last_to;
+  Point start_at;
+
+  GdkGC *gc;
+
+  int x1, y1, x2, y2;
+  Point start_box;
+  Point end_box;
+
+  /* Undo info: */
+  Point *orig_pos;
 };
 
 extern Tool *create_modify_tool(void);
