@@ -206,7 +206,7 @@ prefs_save(void)
   strncat(filename, "/.dia/diarc",512);
   filename[511] = 0;
   
-  file = fopen(filename, "wt");
+  file = fopen(filename, "w");
   
   fprintf(file, "# Note: This file is automatically generated by Dia\n");
   for (i=0;i<NUM_PREFS_DATA;i++) {
