@@ -125,19 +125,28 @@ static RenderOps EpsRenderOps = {
 
 static void print_reencode_font(FILE *file, char *fontname)
 {
-  fprintf(file,
-	  "/%s-latin1\n"
-	  "    /%s findfont\n"
-	  "    dup length dict begin\n"
-	  "	{1 index /FID ne {def} {pop pop} ifelse} forall\n"
-	  "	/Encoding isolatin1encoding def\n"
-	  "    currentdict end\n"
-	  "definefont pop\n", fontname, fontname);
+  /* Don't reencode the Symbol font, as it doesn't work in latin1 encoding.
+   * Instead, just define Symbol-latin1 to be the same as Symbol. */
+  if (!strcmp(fontname, "Symbol"))
+    fprintf(file,
+	    "/%s-latin1\n"
+	    "    /%s findfont\n"
+	    "definefont pop\n", fontname, fontname);
+  else
+    fprintf(file,
+	    "/%s-latin1\n"
+	    "    /%s findfont\n"
+	    "    dup length dict begin\n"
+	    "	{1 index /FID ne {def} {pop pop} ifelse} forall\n"
+	    "	/Encoding isolatin1encoding def\n"
+	    "    currentdict end\n"
+	    "definefont pop\n", fontname, fontname);
 }
 
 
-RendererEPS *
-new_eps_renderer(Diagram *dia, char *filename)
+static RendererEPS *
+create_eps_renderer(DiagramData *data, const char *filename,
+		    const char *diafilename)
 {
   RendererEPS *renderer;
   FILE *file;
@@ -167,7 +176,7 @@ new_eps_renderer(Diagram *dia, char *filename)
   renderer->saved_line_style = LINESTYLE_SOLID;
   
   time_now  = time(NULL);
-  extent = &dia->data->extents;
+  extent = &data->extents;
   
   scale = 28.346;
   
@@ -188,7 +197,7 @@ new_eps_renderer(Diagram *dia, char *filename)
 	  "%%%%BeginSetup\n"
 	  "%%%%EndSetup\n"
 	  "%%%%EndComments\n",
-	  dia->filename,
+	  diafilename,
 	  VERSION,
 	  ctime(&time_now),
 	  name,
@@ -358,6 +367,12 @@ new_eps_renderer(Diagram *dia, char *filename)
   return renderer;
 }
 
+RendererEPS *
+new_eps_renderer(Diagram *dia, gchar *filename)
+{
+  return create_eps_renderer(dia->data, filename, dia->filename);
+}
+
 void
 destroy_eps_renderer(RendererEPS *renderer)
 {
@@ -1070,3 +1085,20 @@ draw_image(RendererEPS *renderer,
   fprintf(renderer->file, "\n");
 }
 
+/* --- export filter interface --- */
+static void
+export_eps(DiagramData *data, const gchar *filename, const gchar *diafilename)
+{
+  RendererEPS *renderer;
+
+  renderer = create_eps_renderer(data, filename, diafilename);
+  data_render(data, (Renderer *)renderer, NULL, NULL, NULL);
+  destroy_eps_renderer(renderer);
+}
+
+static const gchar *extensions[] = { "eps", "epsi", NULL };
+DiaExportFilter eps_export_filter = {
+  N_("Encapsulated Postscript"),
+  extensions,
+  export_eps
+};
