@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,18 +17,13 @@
  */
 
 #include <math.h>
-
-#include <gtk/gtkfeatures.h>
-
-#ifndef GTK_HAVE_FEATURES_1_1_0 
-#define Font XXXFont /* Ugly namespace collision... */
-#include <gdk/gdkx.h>
-#undef Font
-#endif
+#include <gdk/gdk.h>
 
 #include "render_gdk.h"
 #include "message.h"
 
+static void begin_render(RendererGdk *renderer, DiagramData *data);
+static void end_render(RendererGdk *renderer);
 static void set_linewidth(RendererGdk *renderer, real linewidth);
 static void set_linecaps(RendererGdk *renderer, LineCaps mode);
 static void set_linejoin(RendererGdk *renderer, LineJoin mode);
@@ -73,12 +68,12 @@ static void fill_ellipse(RendererGdk *renderer,
 			 real width, real height,
 			 Color *color);
 static void draw_bezier(RendererGdk *renderer, 
-			Point *points,
-			int numpoints, /* numpoints = 4+3*n, n=>0 */
+			BezPoint *points,
+			int numpoints,
 			Color *color);
 static void fill_bezier(RendererGdk *renderer, 
-			Point *points, /* Last point must be same as first point */
-			int numpoints, /* numpoints = 4+3*n, n=>0 */
+			BezPoint *points, /* Last point must be same as first point */
+			int numpoints,
 			Color *color);
 static void draw_string(RendererGdk *renderer,
 			const char *text,
@@ -87,13 +82,32 @@ static void draw_string(RendererGdk *renderer,
 static void draw_image(RendererGdk *renderer,
 		       Point *point,
 		       real width, real height,
-		       void *not_decided_yet);
+		       DiaImage image);
 
 static real get_text_width(RendererGdk *renderer,
 			   const char *text, int length);
 
+static void clip_region_clear(RendererGdk *renderer);
+static void clip_region_add_rect(RendererGdk *renderer,
+				 Rectangle *rect);
+
+static void draw_pixel_line(RendererGdk *renderer,
+			    int x1, int y1,
+			    int x2, int y2,
+			    Color *color);
+static void draw_pixel_rect(RendererGdk *renderer,
+				 int x, int y,
+				 int width, int height,
+				 Color *color);
+static void fill_pixel_rect(RendererGdk *renderer,
+				 int x, int y,
+				 int width, int height,
+				 Color *color);
 
 static RenderOps GdkRenderOps = {
+  (BeginRenderFunc) begin_render,
+  (EndRenderFunc) end_render,
+
   (SetLineWidthFunc) set_linewidth,
   (SetLineCapsFunc) set_linecaps,
   (SetLineJoinFunc) set_linejoin,
@@ -126,7 +140,14 @@ static RenderOps GdkRenderOps = {
 };
 
 static InteractiveRenderOps GdkInteractiveRenderOps = {
-  (GetTextWidthFunc) get_text_width
+  (GetTextWidthFunc) get_text_width,
+
+  (ClipRegionClearFunc) clip_region_clear,
+  (ClipRegionAddRectangleFunc) clip_region_add_rect,
+
+  (DrawPixelLineFunc) draw_pixel_line,
+  (DrawPixelRectangleFunc) draw_pixel_rect,
+  (FillPixelRectangleFunc) fill_pixel_rect,
 };
 
 RendererGdk *
@@ -139,26 +160,94 @@ new_gdk_renderer(DDisplay *ddisp)
   renderer->renderer.is_interactive = 1;
   renderer->renderer.interactive_ops = &GdkInteractiveRenderOps;
   renderer->ddisp = ddisp;
-  renderer->render_gc = gdk_gc_new(ddisp->pixmap);
+  renderer->render_gc = NULL;
+
+  renderer->pixmap = NULL;
+  renderer->renderer.pixel_width = 0;
+  renderer->renderer.pixel_height = 0;
+  renderer->clip_region = NULL;
 
   renderer->line_width = 1;
   renderer->line_style = GDK_LINE_SOLID;
   renderer->cap_style = GDK_CAP_BUTT;
   renderer->join_style = GDK_JOIN_MITER;
   
-  gdk_gc_set_line_attributes(renderer->render_gc,
-			     renderer->line_width,
-			     renderer->line_style,
-			     renderer->cap_style,
-			     renderer->join_style);
-
   renderer->saved_line_style = LINESTYLE_SOLID;
   renderer->dash_length = 10;
-  renderer->dot_length = 1;
+  renderer->dot_length = 2;
   
   return renderer;
 }
 
+void
+destroy_gdk_renderer(RendererGdk *renderer)
+{
+  if (renderer->pixmap != NULL)
+    gdk_pixmap_unref(renderer->pixmap);
+
+  if (renderer->render_gc != NULL)
+    gdk_gc_unref(renderer->render_gc);
+
+  if (renderer->clip_region != NULL)
+    gdk_region_destroy(renderer->clip_region);
+
+  g_free(renderer);
+}
+
+void
+gdk_renderer_set_size(RendererGdk *renderer, GdkWindow *window,
+		      int width, int height)
+{
+  
+  if (renderer->pixmap != NULL) {
+    gdk_pixmap_unref(renderer->pixmap);
+  }
+
+  renderer->pixmap = gdk_pixmap_new(window,  width, height, -1);
+  renderer->renderer.pixel_width = width;
+  renderer->renderer.pixel_height = height;
+
+  if (renderer->render_gc == NULL) {
+    renderer->render_gc = gdk_gc_new(renderer->pixmap);
+
+    gdk_gc_set_line_attributes(renderer->render_gc,
+			       renderer->line_width,
+			       renderer->line_style,
+			       renderer->cap_style,
+			       renderer->join_style);
+  }
+}
+
+extern void
+renderer_gdk_copy_to_window(RendererGdk *renderer, GdkWindow *window,
+			    int x, int y, int width, int height)
+{
+  static GdkGC *copy_gc = NULL;
+
+  if (copy_gc == NULL) {
+    copy_gc = gdk_gc_new(window);
+  }
+  
+  gdk_draw_pixmap(window,
+		  copy_gc,
+		  renderer->pixmap,
+		  x, y,
+		  x, y,
+		  width, height);
+}
+
+static void
+begin_render(RendererGdk *renderer, DiagramData *data)
+{
+}
+
+static void
+end_render(RendererGdk *renderer)
+{
+}
+
+
+
 static void
 set_linewidth(RendererGdk *renderer, real linewidth)
 {  /* 0 == hairline **/
@@ -234,12 +323,7 @@ set_linestyle(RendererGdk *renderer, LineStyle mode)
     renderer->line_style = GDK_LINE_ON_OFF_DASH;
     dash_list[0] = renderer->dash_length;
     dash_list[1] = renderer->dash_length;
-#ifndef GTK_HAVE_FEATURES_1_1_0 
-    XSetDashes(GDK_DISPLAY(), GDK_GC_XGC(renderer->render_gc),
-	       0, dash_list, 2);
-#else
     gdk_gc_set_dashes(renderer->render_gc, 0, dash_list, 2);
-#endif
     break;
   case LINESTYLE_DASH_DOT:
     renderer->line_style = GDK_LINE_ON_OFF_DASH;
@@ -250,12 +334,7 @@ set_linestyle(RendererGdk *renderer, LineStyle mode)
     dash_list[1] = hole_width;
     dash_list[2] = renderer->dot_length;
     dash_list[3] = hole_width;
-#ifndef GTK_HAVE_FEATURES_1_1_0 
-    XSetDashes(GDK_DISPLAY(), GDK_GC_XGC(renderer->render_gc),
-	       0, dash_list, 4);
-#else
     gdk_gc_set_dashes(renderer->render_gc, 0, dash_list, 4);
-#endif
     break;
   case LINESTYLE_DASH_DOT_DOT:
     renderer->line_style = GDK_LINE_ON_OFF_DASH;
@@ -268,12 +347,13 @@ set_linestyle(RendererGdk *renderer, LineStyle mode)
     dash_list[3] = hole_width;
     dash_list[4] = renderer->dot_length;
     dash_list[5] = hole_width;
-#ifndef GTK_HAVE_FEATURES_1_1_0 
-    XSetDashes(GDK_DISPLAY(), GDK_GC_XGC(renderer->render_gc),
-	       0, dash_list, 6);
-#else
     gdk_gc_set_dashes(renderer->render_gc, 0, dash_list, 6);
-#endif
+    break;
+  case LINESTYLE_DOTTED:
+    renderer->line_style = GDK_LINE_ON_OFF_DASH;
+    dash_list[0] = renderer->dot_length;
+    dash_list[1] = renderer->dot_length;
+    gdk_gc_set_dashes(renderer->render_gc, 0, dash_list, 2);
     break;
   }
   gdk_gc_set_line_attributes(renderer->render_gc,
@@ -341,7 +421,7 @@ draw_line(RendererGdk *renderer,
   color_convert(line_color, &color);
   gdk_gc_set_foreground(gc, &color);
   
-  gdk_draw_line(ddisp->pixmap, gc,
+  gdk_draw_line(renderer->pixmap, gc,
 		x1, y1,	x2, y2);
 }
 
@@ -367,7 +447,7 @@ draw_polyline(RendererGdk *renderer,
   color_convert(line_color, &color);
   gdk_gc_set_foreground(gc, &color);
   
-  gdk_draw_lines(ddisp->pixmap, gc, gdk_points, num_points);
+  gdk_draw_lines(renderer->pixmap, gc, gdk_points, num_points);
   g_free(gdk_points);
 }
 
@@ -393,7 +473,7 @@ draw_polygon(RendererGdk *renderer,
   color_convert(line_color, &color);
   gdk_gc_set_foreground(gc, &color);
   
-  gdk_draw_polygon(ddisp->pixmap, gc, FALSE, gdk_points, num_points);
+  gdk_draw_polygon(renderer->pixmap, gc, FALSE, gdk_points, num_points);
   g_free(gdk_points);
 }
 
@@ -419,7 +499,7 @@ fill_polygon(RendererGdk *renderer,
   color_convert(line_color, &color);
   gdk_gc_set_foreground(gc, &color);
   
-  gdk_draw_polygon(ddisp->pixmap, gc, TRUE, gdk_points, num_points);
+  gdk_draw_polygon(renderer->pixmap, gc, TRUE, gdk_points, num_points);
   g_free(gdk_points);
 }
 
@@ -444,7 +524,7 @@ draw_rect(RendererGdk *renderer,
   color_convert(color, &gdkcolor);
   gdk_gc_set_foreground(gc, &gdkcolor);
 
-  gdk_draw_rectangle (ddisp->pixmap,
+  gdk_draw_rectangle (renderer->pixmap,
 		      gc, FALSE,
 		      left, top,
 		      right-left,
@@ -472,7 +552,7 @@ fill_rect(RendererGdk *renderer,
   color_convert(color, &gdkcolor);
   gdk_gc_set_foreground(gc, &gdkcolor);
 
-  gdk_draw_rectangle (ddisp->pixmap,
+  gdk_draw_rectangle (renderer->pixmap,
 		      gc, TRUE,
 		      left, top,
 		      right-left,
@@ -509,7 +589,7 @@ draw_arc(RendererGdk *renderer,
   if (dangle<0)
     dangle += 360.0;
   
-  gdk_draw_arc(ddisp->pixmap,
+  gdk_draw_arc(renderer->pixmap,
 	       gc, FALSE,
 	       left, top, right-left, bottom-top,
 	       (int) (angle1*64.0), (int) (dangle*64.0));
@@ -545,7 +625,7 @@ fill_arc(RendererGdk *renderer,
   if (dangle<0)
     dangle += 360.0;
   
-  gdk_draw_arc(ddisp->pixmap,
+  gdk_draw_arc(renderer->pixmap,
 	       gc, TRUE,
 	       left, top, right-left, bottom-top,
 	       (int) (angle1*64), (int) (dangle*64));
@@ -610,6 +690,7 @@ bezier_add_lines(DDisplay *ddisp,
   Point s[4];
   Point middle;
   coord delta;
+  real v_len_sq;
 
   /* Check if almost flat: */
   u = points[1];
@@ -617,7 +698,10 @@ bezier_add_lines(DDisplay *ddisp,
   v = points[3];
   point_sub(&v, &points[0]);
   y = v;
-  point_scale(&y, point_dot(&u,&v)/point_dot(&v,&v));
+  v_len_sq = point_dot(&v,&v);
+  if (v_len_sq < 0.000001)
+    v_len_sq = 0.000001;
+  point_scale(&y, point_dot(&u,&v)/v_len_sq);
   x = u;
   point_sub(&x,&y);
   delta = ddisplay_transform_length(ddisp, point_dot(&x,&x));
@@ -627,11 +711,14 @@ bezier_add_lines(DDisplay *ddisp,
     v = points[0];
     point_sub(&v, &points[3]);
     y = v;
-    point_scale(&y, point_dot(&u,&v)/point_dot(&v,&v));
+    v_len_sq = point_dot(&v,&v);
+    if (v_len_sq < 0.000001)
+      v_len_sq = 0.000001;
+    point_scale(&y, point_dot(&u,&v)/v_len_sq);
     x = u;
     point_sub(&x,&y);
     delta = ddisplay_transform_length(ddisp, point_dot(&x,&x));
-    if (delta < BEZIER_SUBDIVIDE_LIMIT_SQ) {
+    if (delta < BEZIER_SUBDIVIDE_LIMIT_SQ) { /* Almost flat, draw a line */
       bezier_add_point(ddisp, bezier, &points[3]);
       return;
     }
@@ -672,17 +759,34 @@ bezier_add_lines(DDisplay *ddisp,
   bezier_add_lines(ddisp, s, bezier);
 }
 
+static void
+bezier_add_curve(DDisplay *ddisp,
+		 Point points[4],
+		 struct bezier_curve *bezier)
+{
+  /* Is the bezier curve malformed? */
+  if ( (distance_point_point(&points[0], &points[1]) < 0.00001) &&
+       (distance_point_point(&points[2], &points[3]) < 0.00001) &&
+       (distance_point_point(&points[0], &points[3]) < 0.00001)) {
+    bezier_add_point(ddisp, bezier, &points[3]);
+  }
+  
+  bezier_add_lines(ddisp, points, bezier);
+}
+
+
 static struct bezier_curve bezier = { NULL, 0, 0 };
 
 static void
 draw_bezier(RendererGdk *renderer, 
-	    Point *points,
+	    BezPoint *points,
 	    int numpoints, /* numpoints = 4+3*n, n=>0 */
 	    Color *color)
 {
   DDisplay *ddisp = renderer->ddisp;
   GdkGC *gc = renderer->render_gc;
   GdkColor gdk_color;
+  Point curve[4];
   int i;
   
   if (bezier.gdk_points == NULL) {
@@ -691,13 +795,29 @@ draw_bezier(RendererGdk *renderer,
   }
 
   bezier.currpoint = 0;
-  
-  bezier_add_point(renderer->ddisp, &bezier, &points[0]);
-  i = 0;
-  while (i<=numpoints-3) {
-    bezier_add_lines(ddisp, &points[i], &bezier);
-    i += 3;
-  }
+
+  if (points[0].type != BEZ_MOVE_TO)
+    g_warning("first BezPoint must be a BEZ_MOVE_TO");
+  curve[3] = points[0].p1;
+  bezier_add_point(renderer->ddisp, &bezier, &points[0].p1);
+  for (i = 1; i < numpoints; i++)
+    switch (points[i].type) {
+    case BEZ_MOVE_TO:
+      g_warning("only first BezPoint can be a BEZ_MOVE_TO");
+      curve[3] = points[i].p1;
+      break;
+    case BEZ_LINE_TO:
+      bezier_add_point(renderer->ddisp, &bezier, &points[i].p1);
+      curve[3] = points[i].p1;
+      break;
+    case BEZ_CURVE_TO:
+      curve[0] = curve[3];
+      curve[1] = points[i].p1;
+      curve[2] = points[i].p2;
+      curve[3] = points[i].p3;
+      bezier_add_curve(ddisp, curve, &bezier);
+      break;
+    }
   
   color_convert(color, &gdk_color);
   gdk_gc_set_foreground(gc, &gdk_color);
@@ -708,7 +828,7 @@ draw_bezier(RendererGdk *renderer,
 			     renderer->cap_style,
 			     GDK_JOIN_ROUND);
 
-  gdk_draw_lines(ddisp->pixmap, gc, bezier.gdk_points, bezier.currpoint);
+  gdk_draw_lines(renderer->pixmap, gc, bezier.gdk_points, bezier.currpoint);
 
   gdk_gc_set_line_attributes(renderer->render_gc,
 			     renderer->line_width,
@@ -719,13 +839,14 @@ draw_bezier(RendererGdk *renderer,
 
 static void
 fill_bezier(RendererGdk *renderer, 
-	    Point *points, /* Last point must be same as first point */
+	    BezPoint *points, /* Last point must be same as first point */
 	    int numpoints, /* numpoints = 4+3*n, n=>0 */
 	    Color *color)
 {
   DDisplay *ddisp = renderer->ddisp;
   GdkGC *gc = renderer->render_gc;
   GdkColor gdk_color;
+  Point curve[4];
   int i;
   
   if (bezier.gdk_points == NULL) {
@@ -735,17 +856,33 @@ fill_bezier(RendererGdk *renderer,
 
   bezier.currpoint = 0;
   
-  bezier_add_point(renderer->ddisp, &bezier, &points[0]);
-  i = 0;
-  while (i<=numpoints-3) {
-    bezier_add_lines(ddisp, &points[i], &bezier);
-    i += 3;
-  }
+  if (points[0].type != BEZ_MOVE_TO)
+    g_warning("first BezPoint must be a BEZ_MOVE_TO");
+  curve[3] = points[0].p1;
+  bezier_add_point(renderer->ddisp, &bezier, &points[0].p1);
+  for (i = 1; i < numpoints; i++)
+    switch (points[i].type) {
+    case BEZ_MOVE_TO:
+      g_warning("only first BezPoint can be a BEZ_MOVE_TO");
+      curve[3] = points[i].p1;
+      break;
+    case BEZ_LINE_TO:
+      bezier_add_point(renderer->ddisp, &bezier, &points[i].p1);
+      curve[3] = points[i].p1;
+      break;
+    case BEZ_CURVE_TO:
+      curve[0] = curve[3];
+      curve[1] = points[i].p1;
+      curve[2] = points[i].p2;
+      curve[3] = points[i].p3;
+      bezier_add_curve(ddisp, curve, &bezier);
+      break;
+    }
   
   color_convert(color, &gdk_color);
   gdk_gc_set_foreground(gc, &gdk_color);
   
-  gdk_draw_polygon(ddisp->pixmap, gc, TRUE,
+  gdk_draw_polygon(renderer->pixmap, gc, TRUE,
 		   bezier.gdk_points, bezier.currpoint);
 }
 
@@ -780,7 +917,7 @@ draw_string(RendererGdk *renderer,
   color_convert(color, &gdkcolor);
   gdk_gc_set_foreground(gc, &gdkcolor);
       
-  gdk_draw_string(ddisp->pixmap,
+  gdk_draw_string(renderer->pixmap,
 		  renderer->gdk_font, gc,
 		  x,y, text);
 }
@@ -789,9 +926,17 @@ static void
 draw_image(RendererGdk *renderer,
 	   Point *point,
 	   real width, real height,
-	   void *not_decided_yet)
+	   DiaImage image)
 {
-  message_error("gkd_renderer: Images are not supported yet!\n");
+  int real_width, real_height, real_x, real_y;
+  
+  real_width = ddisplay_transform_length(renderer->ddisp, width);
+  real_height = ddisplay_transform_length(renderer->ddisp, height);
+  ddisplay_transform_coords(renderer->ddisp, point->x, point->y,
+			    &real_x, &real_y);
+
+  dia_image_draw(image,  renderer->pixmap, real_x, real_y,
+		 real_width, real_height);
 }
 
 static real
@@ -805,3 +950,90 @@ get_text_width(RendererGdk *renderer,
   return ddisplay_untransform_length(renderer->ddisp, (real) iwidth);
 }
 
+
+static void
+clip_region_clear(RendererGdk *renderer)
+{
+  if (renderer->clip_region != NULL)
+    gdk_region_destroy(renderer->clip_region);
+
+  renderer->clip_region =  gdk_region_new();
+
+  gdk_gc_set_clip_region(renderer->render_gc, renderer->clip_region);
+}
+
+static void
+clip_region_add_rect(RendererGdk *renderer,
+		     Rectangle *rect)
+{
+  DDisplay *ddisp = renderer->ddisp;
+  GdkRegion *old_reg;
+  GdkRectangle clip_rect;
+  int x1,y1;
+  int x2,y2;
+
+  ddisplay_transform_coords(ddisp, rect->left, rect->top,  &x1, &y1);
+  ddisplay_transform_coords(ddisp, rect->right, rect->bottom,  &x2, &y2);
+  
+  clip_rect.x = x1;
+  clip_rect.y = y1;
+  clip_rect.width = x2 - x1 + 1;
+  clip_rect.height = y2 - y1 + 1;
+
+  old_reg = renderer->clip_region;
+
+  renderer->clip_region =
+    gdk_region_union_with_rect( renderer->clip_region, &clip_rect );
+  
+  gdk_region_destroy(old_reg);
+
+  gdk_gc_set_clip_region(renderer->render_gc, renderer->clip_region);
+}
+
+static void
+draw_pixel_line(RendererGdk *renderer,
+		int x1, int y1,
+		int x2, int y2,
+		Color *color)
+{
+  GdkGC *gc = renderer->render_gc;
+  GdkColor gdkcolor;
+  
+  color_convert(color, &gdkcolor);
+  gdk_gc_set_foreground(gc, &gdkcolor);
+  
+  gdk_draw_line(renderer->pixmap, gc, x1, y1, x2, y2);
+}
+
+static void
+draw_pixel_rect(RendererGdk *renderer,
+		int x, int y,
+		int width, int height,
+		Color *color)
+{
+  GdkGC *gc = renderer->render_gc;
+  GdkColor gdkcolor;
+    
+  color_convert(color, &gdkcolor);
+  gdk_gc_set_foreground(gc, &gdkcolor);
+
+  gdk_draw_rectangle (renderer->pixmap, gc, FALSE,
+		      x, y,  width, height);
+}
+
+static void
+fill_pixel_rect(RendererGdk *renderer,
+		int x, int y,
+		int width, int height,
+		Color *color)
+{
+  GdkGC *gc = renderer->render_gc;
+  GdkColor gdkcolor;
+    
+  color_convert(color, &gdkcolor);
+  gdk_gc_set_foreground(gc, &gdkcolor);
+
+  gdk_draw_rectangle (renderer->pixmap, gc, TRUE,
+		      x, y,  width, height);
+}
+
