@@ -129,7 +129,7 @@ static RenderOps SvgRenderOps = {
 };
 
 RendererSVG *
-new_svg_renderer(Diagram *dia, char *filename)
+new_svg_renderer(DiagramData *data, const char *filename)
 {
   RendererSVG *renderer;
   FILE *file;
@@ -138,7 +138,7 @@ new_svg_renderer(Diagram *dia, char *filename)
   Rectangle *extent;
   char *name;
  
-  file = fopen(filename, "wt");
+  file = fopen(filename, "w");
 
   if (file==NULL) {
     message_error(_("Couldn't open: '%s' for writing.\n"), filename);
@@ -161,13 +161,13 @@ new_svg_renderer(Diagram *dia, char *filename)
   renderer->doc = xmlNewDoc("1.0");
   renderer->doc->standalone = FALSE;
   xmlCreateIntSubset(renderer->doc, "svg",
-		     "-//W3C//DTD SVG July 1999//EN",
-		     "http://www.w3.org/Graphics/SVG/svg-19990730.dtd");
+		     "-//W3C//DTD SVG December 1999//EN",
+		     "http://www.w3.org/Graphics/SVG/svg-19991203.dtd");
   renderer->root = xmlNewDocNode(renderer->doc, NULL, "svg", NULL);
   renderer->doc->root = renderer->root;
 
   /* set the extents of the SVG document */
-  extent = &dia->data->extents;
+  extent = &data->extents;
   g_snprintf(buf, sizeof(buf), "%dcm",
 	     (int)ceil((extent->right - extent->left)));
   xmlSetProp(renderer->root, "width", buf);
@@ -177,14 +177,15 @@ new_svg_renderer(Diagram *dia, char *filename)
   g_snprintf(buf, sizeof(buf), "%d %d %d %d",
 	     (int)floor(extent->left), (int)floor(extent->top),
 	     (int)ceil(extent->right), (int)ceil(extent->bottom));
-  xmlSetProp(renderer->root, "fitBoxToViewport", buf);
+  xmlSetProp(renderer->root, "viewBox", buf);
   
   time_now = time(NULL);
   name = getlogin();
   if (name==NULL)
     name = "a user";
 
-  /* some comments at the top of the file ... * /
+#if 0
+  /* some comments at the top of the file ... */
   xmlAddChild(renderer->root, xmlNewText("\n"));
   xmlAddChild(renderer->root, xmlNewComment("Dia-Version: "VERSION));
   xmlAddChild(renderer->root, xmlNewText("\n"));
@@ -192,7 +193,7 @@ new_svg_renderer(Diagram *dia, char *filename)
   xmlAddChild(renderer->root, xmlNewComment(buf));
   xmlAddChild(renderer->root, xmlNewText("\n"));
   g_snprintf(buf, sizeof(buf), "Date: %s", ctime(&time_now));
-  buf[strlen(buf)-1] = '\0'; / * remove the trailing new line * /
+  buf[strlen(buf)-1] = '\0'; /* remove the trailing new line */
   xmlAddChild(renderer->root, xmlNewComment(buf));
   xmlAddChild(renderer->root, xmlNewText("\n"));
   g_snprintf(buf, sizeof(buf), "For: %s", name);
@@ -200,7 +201,7 @@ new_svg_renderer(Diagram *dia, char *filename)
   xmlAddChild(renderer->root, xmlNewText("\n\n"));
 
   xmlNewChild(renderer->root, NULL, "title", dia->filename);
-  */
+#endif
   
   return renderer;
 }
@@ -818,3 +819,19 @@ draw_image(RendererSVG *renderer,
   */
 }
 
+static void
+export_svg(DiagramData *data, const gchar *filename, const gchar *diafilename)
+{
+  RendererSVG *renderer;
+
+  renderer = new_svg_renderer(data, filename);
+  data_render(data, (Renderer *)renderer, NULL, NULL, NULL);
+  destroy_svg_renderer(renderer);
+}
+
+static const gchar *extensions[] = { "svg", NULL };
+DiaExportFilter svg_export_filter = {
+  N_("Scalable Vector Graphics"),
+  extensions,
+  export_svg
+};
