@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,14 +21,24 @@
 #include "geometry.h"
 #include "render.h"
 
+/* NOTE: Add new arrow types at the end, or the enums
+   will change order leading to file incompatibilities. */
+   
 typedef enum {
+  ARROW_NONE,
   ARROW_LINES,
   ARROW_HOLLOW_TRIANGLE,
   ARROW_FILLED_TRIANGLE,
   ARROW_HOLLOW_DIAMOND,
-  ARROW_FILLED_DIAMOND
+  ARROW_FILLED_DIAMOND,
+  ARROW_HALF_HEAD
 } ArrowType;
 
+typedef struct {
+  ArrowType type;
+  real length;
+  real width;
+} Arrow;
 
 extern void arrow_draw(Renderer *renderer, ArrowType type,
 		       Point *to, Point *from,
