@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,7 @@ struct _Color {
 
 extern void color_init(void);
 extern void color_convert(Color *color, GdkColor *gdkcolor);
+extern gboolean color_equals(Color *color1, Color *color2);
 
 extern Color color_black, color_white;
 extern GdkColor color_gdk_black, color_gdk_white;
