@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,10 @@
 #include "orth_conn.h"
 #include "arrows.h"
 #include "utils.h"
+#include "poly_conn.h"
+#include "bezier_conn.h"
+#include "widgets.h"
+#include "intl.h"
 
 /* This is a file with dummy dependencies so that all
    object files will be linked into the app.
@@ -33,10 +37,15 @@ static void *dummy_dep[] = {
   connection_move_handle,
   element_update_boundingbox,
   orthconn_update_data,
+  polyconn_init,
+  bezierconn_init,
   new_text,
   font_getfont,
   new_render_store,
   new_render_object,
   nearest_pow,
-  arrow_draw
+  arrow_draw,
+  dia_font_selector_new, /* widgets.o */
+  new_object_state_change, /* objchange.o */
+  intl_score_locale /* intl.o */
 };
