@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -106,6 +106,33 @@ point_in_rectangle(Rectangle* r, Point *p)
   return TRUE;
 }
 
+int
+rectangle_in_rectangle(Rectangle* outer, Rectangle *inner)
+{
+  if ( (inner->left < outer->left) ||
+       (inner->right > outer->right) ||
+       (inner->top < outer->top) ||
+       (inner->bottom > outer->bottom))
+    return FALSE;
+  
+  return TRUE;
+}
+
+void
+rectangle_add_point(Rectangle *r, Point *p)
+{
+  if (p->x < r->left)
+    r->left = p->x;
+  else if (p->x > r->right)
+    r->right = p->x;
+
+  if (p->y < r->top)
+    r->top = p->y;
+  else if (p->y > r->bottom)
+    r->bottom = p->y;
+}
+
+
 real
 distance_point_point(Point *p1, Point *p2)
 {
@@ -121,6 +148,7 @@ distance_point_point_manhattan(Point *p1, Point *p2)
   real dy = p1->y - p2->y;
   return ABS(dx) + ABS(dy);
 }
+
 /*
  * This function estimates the distance from a point to a rectangle.
  * If the point is in the rectangle, 0.0 is returned. Otherwise the
@@ -151,9 +179,11 @@ distance_rectangle_point(Rectangle *rect, Point *point)
 /*
  * This function estimates the distance from a point to a line segment
  * specified by two endpoints.
- * If the point is on the line, 0.0 is returned. Otherwise the
+ * If the point is on the line segment, 0.0 is returned. Otherwise the
  * distance in the R^2 metric from the point to the nearest point
  * on the line segment is returned. Does one sqrt per call.
+ * Philosophical bug: line_width is ignored iff point is beyond
+ * end of line segment.
  */
 real
 distance_line_point(Point *line_start, Point *line_end,
