@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,11 +26,13 @@ typedef struct _SheetObject SheetObject;
 #include "object.h"
 
 struct _SheetObject {
-  ObjectType *type;
+  char *object_type;
   char *description;
   char **pixmap; /* in xpm format */
 
   void *user_data;
+
+  char *pixmap_file; /* fallback if pixmap is NULL */
 };
 
 struct _Sheet {
@@ -46,4 +48,6 @@ extern void sheet_append_sheet_obj(Sheet *sheet, SheetObject *type);
 extern void register_sheet(Sheet *sheet);
 extern GSList *get_sheets_list(void);
 
+extern void load_all_sheets(void);
+
 #endif /* SHEET_H */
