@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,29 @@
 #include "message.h"
 
 static int text_key_event(Focus *focus, guint keysym,
-			  char *str, int strlen);
+			  char *str, int strlen,
+			  ObjectChange **change);
+
+enum change_type {
+  TYPE_DELETE_BACKWARD,
+  TYPE_DELETE_FORWARD,
+  TYPE_INSERT_CHAR,
+  TYPE_JOIN_ROW,
+  TYPE_SPLIT_ROW
+};
+
+struct TextObjectChange {
+  ObjectChange obj_change;
+
+  Text *text;
+  enum change_type type;
+  char ch;
+  int pos;
+  int row;
+};
+
+static ObjectChange *text_create_change(Text *text, enum change_type type,
+					char ch, int pos, int row);
 
 static void
 calc_width(Text *text)
@@ -81,15 +103,16 @@ set_string(Text *text, const char *string)
   int len;
   const char *s,*s2;
   
-   s = string;
+  s = string;
   
   numlines = 1;
-  while ( (s = strchr(s,'\n')) != NULL ) {
-    s++;
-    if ((*s) != 0) {
-      numlines++;
+  if (s != NULL) 
+    while ( (s = strchr(s,'\n')) != NULL ) {
+	s++;
+	if ((*s) != 0) {
+	    numlines++;
+	}
     }
-  }
   text->numlines = numlines;
   text->line = g_malloc(sizeof(char *)*numlines);
   text->strlen = g_malloc(sizeof(int)*numlines);
@@ -97,6 +120,15 @@ set_string(Text *text, const char *string)
   text->row_width = g_malloc(sizeof(real)*numlines);
 
   s = string;
+
+  if (string==NULL) {
+    text->line[0] = (char *)g_malloc(1);
+    text->line[0][0] = 0;
+    text->strlen[0] = 0;
+    text->alloclen[0] = 1;
+    return;
+  }
+
   for (i=0;i<numlines;i++) {
     s2 = strchr(s,'\n');
     if (s2==NULL) {
@@ -397,6 +429,7 @@ text_draw(Text *text, Renderer *renderer)
     p2.x = curs_x;
     p2.y = curs_y + text->height;
     
+    renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
     renderer->ops->set_linewidth(renderer, 0.1);
     renderer->ops->draw_line(renderer, &p1, &p2, &color_black);
   }
@@ -409,6 +442,13 @@ text_grab_focus(Text *text, Object *object)
   request_focus(&text->focus);
 }
 
+void
+text_set_cursor_at_end( Text* text )
+{
+  text->cursor_row = text->numlines - 1 ;
+  text->cursor_pos = text->strlen[text->cursor_row] ;
+}
+
 void
 text_set_cursor(Text *text, Point *clicked_point,
 		Renderer *renderer)
@@ -673,11 +713,15 @@ text_insert_char(Text *text, char c)
 }
 
 static int
-text_key_event(Focus *focus, guint keyval, char *str, int strlen)
+text_key_event(Focus *focus, guint keyval, char *str, int strlen,
+	       ObjectChange **change)
 {
   Text *text;
   int return_val;
-  return_val = TRUE;
+  int row;
+
+  return_val = FALSE;
+  *change = NULL;
   
   text = (Text *)focus->user_data;
 
@@ -715,18 +759,54 @@ text_key_event(Focus *focus, guint keyval, char *str, int strlen)
       text->cursor_pos = text->strlen[text->cursor_row];
     break;
   case GDK_Delete:
+    return_val = TRUE;
+    row = text->cursor_row;
+    if (text->cursor_pos >= text->strlen[row]) {
+      if (row+1 < text->numlines) {
+	*change = text_create_change(text, TYPE_JOIN_ROW, 'Q',
+				     text->cursor_pos, row);
+      } else {
+	return_val = FALSE;
+	break;
+      }
+    } else {
+      *change = text_create_change(text, TYPE_DELETE_FORWARD,
+				   text->line[row][text->cursor_pos],
+				   text->cursor_pos, text->cursor_row);
+    }
     text_delete_forward(text);
     break;
   case GDK_BackSpace:
+    return_val = TRUE;
+    row = text->cursor_row;
+    if (text->cursor_pos <= 0) {
+      if (row > 0) {
+	*change = text_create_change(text, TYPE_JOIN_ROW, 'Q',
+				     text->strlen[row-1], row-1);
+      } else {
+	return_val = FALSE;
+	break;
+      }
+    } else {
+      *change = text_create_change(text, TYPE_DELETE_BACKWARD,
+				   text->line[row][text->cursor_pos-1],
+				   text->cursor_pos-1, text->cursor_row);
+    }
     text_delete_backward(text);
     break;
   case GDK_Return:
+    return_val = TRUE;
+    *change = text_create_change(text, TYPE_SPLIT_ROW, 'Q',
+				 text->cursor_pos, text->cursor_row);
     text_split_line(text);
     break;
   default:
-    return_val = FALSE;
-    if (strlen>0)
+    if (strlen>0) {
+      return_val = TRUE;
+      *change = text_create_change(text, TYPE_INSERT_CHAR, str[0],
+				   text->cursor_pos, text->cursor_row);
       text_insert_char(text, str[0]);
+    }
     break;
   }  
 
@@ -743,3 +823,180 @@ int text_is_empty(Text *text)
   }
   return TRUE;
 }
+
+void
+data_add_text(AttributeNode attr, Text *text)
+{
+  DataNode composite;
+  char *str;
+
+  composite = data_add_composite(attr, "text");
+
+  str = text_get_string_copy(text);
+  data_add_string(composite_add_attribute(composite, "string"),
+		  str);
+  g_free(str);
+  data_add_font(composite_add_attribute(composite, "font"),
+		text->font);
+  data_add_real(composite_add_attribute(composite, "height"),
+		text->height);
+  data_add_point(composite_add_attribute(composite, "pos"),
+		    &text->position);
+  data_add_color(composite_add_attribute(composite, "color"),
+		 &text->color);
+  data_add_enum(composite_add_attribute(composite, "alignment"),
+		text->alignment);
+}
+
+
+Text *
+data_text(AttributeNode text_attr)
+{
+  char *string = "";
+  Font *font;
+  real height;
+  Point pos = {0.0, 0.0};
+  Color col;
+  Alignment align;
+  AttributeNode attr;
+  DataNode composite_node;
+  Text *text;
+
+  composite_node = attribute_first_data(text_attr);
+
+  attr = composite_find_attribute(text_attr, "string");
+  if (attr != NULL)
+    string = data_string(attribute_first_data(attr));
+
+  font = font_getfont("Courier");
+  attr = composite_find_attribute(text_attr, "font");
+  if (attr != NULL)
+    font = data_font(attribute_first_data(attr));
+
+  height = 1.0;
+  attr = composite_find_attribute(text_attr, "height");
+  if (attr != NULL)
+    height = data_real(attribute_first_data(attr));
+
+  attr = composite_find_attribute(text_attr, "pos");
+  if (attr != NULL)
+    data_point(attribute_first_data(attr), &pos);
+
+  col = color_black;
+  attr = composite_find_attribute(text_attr, "color");
+  if (attr != NULL)
+    data_color(attribute_first_data(attr), &col);
+
+  align = ALIGN_LEFT;
+  attr = composite_find_attribute(text_attr, "alignment");
+  if (attr != NULL)
+    align = data_enum(attribute_first_data(attr));
+  
+  text = new_text(string, font, height, &pos, &col, align);
+  if (string) free(string);
+  return text;
+}
+
+void
+text_get_attributes(Text *text, TextAttributes *attr)
+{
+  attr->font = text->font;
+  attr->height = text->height;
+  attr->position = text->position;
+  attr->color = text->color;
+  attr->alignment = text->alignment;
+}
+
+void
+text_set_attributes(Text *text, TextAttributes *attr)
+{
+  text->font = attr->font;
+  text->height = attr->height;
+  text->position = attr->position;
+  text->color = attr->color;
+  text->alignment = attr->alignment;
+}
+
+static void
+text_change_apply(struct TextObjectChange *change, Object *obj)
+{
+  Text *text = change->text;
+  switch (change->type) {
+  case TYPE_INSERT_CHAR:
+    text->cursor_pos = change->pos;
+    text->cursor_row = change->row;
+    text_insert_char(text, change->ch);
+    break;
+  case TYPE_DELETE_BACKWARD:
+    text->cursor_pos = change->pos+1;
+    text->cursor_row = change->row;
+    text_delete_backward(text);
+    break;
+  case TYPE_DELETE_FORWARD:
+    text->cursor_pos = change->pos;
+    text->cursor_row = change->row;
+    text_delete_forward(text);
+    break;
+  case TYPE_SPLIT_ROW:
+    text->cursor_pos = change->pos;
+    text->cursor_row = change->row;
+    text_split_line(text);
+    break;
+  case TYPE_JOIN_ROW:
+    text_join_lines(text, change->row);
+    break;
+  }
+}
+static void
+text_change_revert(struct TextObjectChange *change, Object *obj)
+{
+  Text *text = change->text;
+  switch (change->type) {
+  case TYPE_INSERT_CHAR:
+    text->cursor_pos = change->pos;
+    text->cursor_row = change->row;
+    text_delete_forward(text);
+    break;
+  case TYPE_DELETE_BACKWARD:
+    text->cursor_pos = change->pos;
+    text->cursor_row = change->row;
+    text_insert_char(text, change->ch);
+    break;
+  case TYPE_DELETE_FORWARD:
+    text->cursor_pos = change->pos;
+    text->cursor_row = change->row;
+    text_insert_char(text, change->ch);
+    text->cursor_pos = change->pos;
+    text->cursor_row = change->row;
+    break;
+  case TYPE_SPLIT_ROW:
+    text_join_lines(text, change->row);
+    break;
+  case TYPE_JOIN_ROW:
+    text->cursor_pos = change->pos;
+    text->cursor_row = change->row;
+    text_split_line(text);
+    break;
+  }
+}
+
+
+static ObjectChange *
+text_create_change(Text *text, enum change_type type,
+		   char ch, int pos, int row)
+{
+  struct TextObjectChange *change;
+
+  change = g_new(struct TextObjectChange, 1);
+
+  change->obj_change.apply = (ObjectChangeApplyFunc) text_change_apply;
+  change->obj_change.revert = (ObjectChangeRevertFunc) text_change_revert;
+  change->obj_change.free = (ObjectChangeFreeFunc) NULL;
+
+  change->text = text;
+  change->type = type;
+  change->ch = ch;
+  change->pos = pos;
+  change->row = row;
+  return (ObjectChange *)change;
+}
