@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,20 +20,20 @@
 #include <math.h>
 #include <string.h>
 
+#include "config.h"
+#include "intl.h"
 #include "object.h"
 #include "orth_conn.h"
 #include "render.h"
 #include "attributes.h"
-#include "files.h"
-#include "sheet.h"
 #include "arrows.h"
 
 #include "uml.h"
 
 #include "pixmaps/association.xpm"
-#include "pixmaps/aggregation.xpm"
 
 typedef struct _Association Association;
+typedef struct _AssociationState AssociationState;
 typedef struct _AssociationPropertiesDialog AssociationPropertiesDialog;
 
 typedef enum {
@@ -59,6 +59,21 @@ typedef struct _AssociationEnd {
   AggregateType aggregate; /* Note: Can only be != NONE on ONE side! */
 } AssociationEnd;
 
+struct _AssociationState {
+  ObjectState obj_state;
+
+  char *name;
+  AssociationDirection direction;
+
+  struct {
+    char *role;
+    char *multiplicity;
+    int arrow;
+    AggregateType aggregate;
+  } end[2];
+};
+
+
 struct _Association {
   OrthConn orth;
 
@@ -88,10 +103,6 @@ struct _AssociationPropertiesDialog {
     GtkToggleButton *aggregate;
     GtkToggleButton *composition;
   } end[2];
-
-  
-  ObjectChangedFunc *changed_callback;
-  void *changed_callback_data;
 };
 
 #define ASSOCIATION_WIDTH 0.1
@@ -105,7 +116,7 @@ static real association_distance_from(Association *assoc, Point *point);
 static void association_select(Association *assoc, Point *clicked_point,
 			       Renderer *interactive_renderer);
 static void association_move_handle(Association *assoc, Handle *handle,
-				    Point *to, HandleMoveReason reason);
+				    Point *to, HandleMoveReason reason, ModifierKeys modifiers);
 static void association_move(Association *assoc, Point *to);
 static void association_draw(Association *assoc, Renderer *renderer);
 static Object *association_create(Point *startpoint,
@@ -114,12 +125,19 @@ static Object *association_create(Point *startpoint,
 				  Handle **handle2);
 static void association_destroy(Association *assoc);
 static Object *association_copy(Association *assoc);
-static void association_show_properties(Association *assoc,
-					ObjectChangedFunc *changed_callback,
-					void *changed_callback_data);
+static GtkWidget *association_get_properties(Association *assoc);
+static ObjectChange *association_apply_properties(Association *assoc);
+static DiaMenu *association_get_object_menu(Association *assoc,
+					    Point *clickedpoint);
 
-static void association_save(Association *assoc, int fd);
-static Object *association_load(int fd, int version);
+static AssociationState *association_get_state(Association *assoc);
+static void association_set_state(Association *assoc,
+				  AssociationState *state);
+
+static void association_save(Association *assoc, ObjectNode obj_node,
+			     const char *filename);
+static Object *association_load(ObjectNode obj_node, int version,
+				const char *filename);
 
 static void association_update_data(Association *assoc);
 
@@ -139,36 +157,17 @@ ObjectType association_type =
   &association_type_ops       /* ops */
 };
 
-SheetObject association_sheetobj =
-{
-  &association_type,             /* type */
-  "Association, two classes are associated.",
-                              /* description */
-  (char **) association_xpm,     /* pixmap */
-
-  GINT_TO_POINTER(0)                        /* user_data */
-};
-
-SheetObject aggregation_sheetobj =
-{
-  &association_type,             /* type */
-  "Aggregation, one class is part of another.",
-                              /* description */
-  (char **) aggregation_xpm,     /* pixmap */
-
-  GINT_TO_POINTER(1)                        /* user_data */
-};
-
 static ObjectOps association_ops = {
-  (DestroyFunc)        association_destroy,
-  (DrawFunc)           association_draw,
-  (DistanceFunc)       association_distance_from,
-  (SelectFunc)         association_select,
-  (CopyFunc)           association_copy,
-  (MoveFunc)           association_move,
-  (MoveHandleFunc)     association_move_handle,
-  (ShowPropertiesFunc) association_show_properties,
-  (IsEmptyFunc)        object_return_false
+  (DestroyFunc)         association_destroy,
+  (DrawFunc)            association_draw,
+  (DistanceFunc)        association_distance_from,
+  (SelectFunc)          association_select,
+  (CopyFunc)            association_copy,
+  (MoveFunc)            association_move,
+  (MoveHandleFunc)      association_move_handle,
+  (GetPropertiesFunc)   association_get_properties,
+  (ApplyPropertiesFunc) association_apply_properties,
+  (ObjectMenuFunc)      association_get_object_menu
 };
 
 static real
@@ -187,7 +186,7 @@ association_select(Association *assoc, Point *clicked_point,
 
 static void
 association_move_handle(Association *assoc, Handle *handle,
-		       Point *to, HandleMoveReason reason)
+		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(assoc!=NULL);
   assert(handle!=NULL);
@@ -338,6 +337,86 @@ association_draw(Association *assoc, Renderer *renderer)
   }
 }
 
+static void
+association_state_free(ObjectState *ostate)
+{
+  AssociationState *state = (AssociationState *)ostate;
+  int i;
+  g_free(state->name);
+
+  for (i=0;i<2;i++) {
+    g_free(state->end[i].role);
+    g_free(state->end[i].multiplicity);
+  }
+}
+
+static AssociationState *
+association_get_state(Association *assoc)
+{
+  int i;
+  AssociationEnd *end;
+
+  AssociationState *state = g_new(AssociationState, 1);
+
+  state->obj_state.free = association_state_free;
+
+  state->name = g_strdup(assoc->name);
+  state->direction = assoc->direction;
+  
+  for (i=0;i<2;i++) {
+    end = &assoc->end[i];
+    state->end[i].role = g_strdup(end->role);
+    state->end[i].multiplicity = g_strdup(end->multiplicity);
+    state->end[i].arrow = end->arrow;
+    state->end[i].aggregate = end->aggregate;
+  }
+
+  return state;
+}
+
+static void
+association_set_state(Association *assoc, AssociationState *state)
+{
+  int i;
+  AssociationEnd *end;
+  
+  g_free(assoc->name);
+  assoc->name = state->name;
+  assoc->text_width = 0.0;
+  if (assoc->name != NULL) {
+    assoc->text_width =
+      font_string_width(assoc->name, assoc_font, ASSOCIATION_FONTHEIGHT);
+  } 
+  
+  assoc->direction = state->direction;
+  
+  for (i=0;i<2;i++) {
+    end = &assoc->end[i];
+    g_free(end->role);
+    g_free(end->multiplicity);
+    end->role = state->end[i].role;
+    end->multiplicity = state->end[i].multiplicity;
+    end->arrow = state->end[i].arrow;
+    end->aggregate = state->end[i].aggregate;
+
+    end->text_width = 0.0;
+    if (end->role != NULL) {
+      end->text_width = 
+	font_string_width(end->role, assoc_font, ASSOCIATION_FONTHEIGHT);
+    }
+    if (end->multiplicity != NULL) {
+      end->text_width =
+	MAX(end->text_width,
+	    font_string_width(end->multiplicity, assoc_font, ASSOCIATION_FONTHEIGHT) );
+    }
+  }
+
+  g_free(state);
+  
+  association_update_data(assoc);
+}
+
+
 static void
 association_update_data(Association *assoc)
 {
@@ -357,8 +436,6 @@ association_update_data(Association *assoc)
   obj->bounding_box.bottom += ASSOCIATION_WIDTH/2.0 + ASSOCIATION_DIAMONDLEN;
   obj->bounding_box.right += ASSOCIATION_WIDTH/2.0 + ASSOCIATION_DIAMONDLEN;
   
-  obj->position = orth->points[0];
-
   /* Calc text pos: */
   num_segm = assoc->orth.numpoints - 1;
   points = assoc->orth.points;
@@ -477,8 +554,6 @@ association_create(Point *startpoint,
 
   orthconn_init(orth, startpoint);
   
-  association_update_data(assoc);
-
   assoc->name = NULL;
   assoc->direction = ASSOC_NODIR;
   for (i=0;i<2;i++) {
@@ -501,13 +576,58 @@ association_create(Point *startpoint,
     assoc->end[1].aggregate = AGGREGATE_NORMAL;
     break;
   }
+
+  association_update_data(assoc);
   
-  *handle1 = &orth->endpoint_handles[0];
-  *handle2 = &orth->endpoint_handles[1];
+  *handle1 = orth->handles[0];
+  *handle2 = orth->handles[orth->numpoints-2];
 
   return (Object *)assoc;
 }
 
+static ObjectChange *
+association_add_segment_callback(Object *obj, Point *clicked, gpointer data)
+{
+  ObjectChange *change;
+  change = orthconn_add_segment((OrthConn *)obj, clicked);
+  association_update_data((Association *)obj);
+  return change;
+}
+
+static ObjectChange *
+association_delete_segment_callback(Object *obj, Point *clicked, gpointer data)
+{
+  ObjectChange *change;
+  change = orthconn_delete_segment((OrthConn *)obj, clicked);
+  association_update_data((Association *)obj);
+  return change;
+}
+
+
+static DiaMenuItem object_menu_items[] = {
+  { N_("Add segment"), association_add_segment_callback, NULL, 1 },
+  { N_("Delete segment"), association_delete_segment_callback, NULL, 1 },
+};
+
+static DiaMenu object_menu = {
+  "Association",
+  sizeof(object_menu_items)/sizeof(DiaMenuItem),
+  object_menu_items,
+  NULL
+};
+
+static DiaMenu *
+association_get_object_menu(Association *assoc, Point *clickedpoint)
+{
+  OrthConn *orth;
+
+  orth = &assoc->orth;
+  /* Set entries sensitive/selected etc here */
+  object_menu_items[0].active = orthconn_can_add_segment(orth, clickedpoint);
+  object_menu_items[1].active = orthconn_can_delete_segment(orth, clickedpoint);
+  return &object_menu;
+}
+
 static void
 association_destroy(Association *assoc)
 {
@@ -515,15 +635,11 @@ association_destroy(Association *assoc)
   
   orthconn_destroy(&assoc->orth);
 
-  if (assoc->name != NULL)
-    g_free(assoc->name);
+  g_free(assoc->name);
 
   for (i=0;i<2;i++) {
-    if (assoc->end[i].role != NULL)
-      g_free(assoc->end[i].role);
-    
-    if (assoc->end[i].multiplicity != NULL)
-      g_free(assoc->end[i].multiplicity);
+    g_free(assoc->end[i].role);
+    g_free(assoc->end[i].multiplicity);
   }
 
   if (assoc->properties_dialog != NULL) {
@@ -568,27 +684,41 @@ association_copy(Association *assoc)
 
 
 static void
-association_save(Association *assoc, int fd)
+association_save(Association *assoc, ObjectNode obj_node,
+		 const char *filename)
 {
   int i;
+  AttributeNode attr;
+  DataNode composite;
   
-  orthconn_save(&assoc->orth, fd);
+  orthconn_save(&assoc->orth, obj_node);
 
-  write_string(fd, assoc->name);
-  write_int32(fd, assoc->direction);
+  data_add_string(new_attribute(obj_node, "name"),
+		  assoc->name);
+  data_add_enum(new_attribute(obj_node, "direction"),
+		assoc->direction);
 
+  attr = new_attribute(obj_node, "ends");
   for (i=0;i<2;i++) {
-    write_string(fd, assoc->end[i].role);
-    write_string(fd, assoc->end[i].multiplicity);
-    write_int32(fd, assoc->end[i].arrow);
-    write_int32(fd, assoc->end[i].aggregate);
+    composite = data_add_composite(attr, NULL);
+
+    data_add_string(composite_add_attribute(composite, "role"),
+		    assoc->end[i].role);
+    data_add_string(composite_add_attribute(composite, "multiplicity"),
+		    assoc->end[i].multiplicity);
+    data_add_boolean(composite_add_attribute(composite, "arrow"),
+		  assoc->end[i].arrow);
+    data_add_enum(composite_add_attribute(composite, "aggregate"),
+		  assoc->end[i].aggregate);
   }
 }
 
 static Object *
-association_load(int fd, int version)
+association_load(ObjectNode obj_node, int version, const char *filename)
 {
   Association *assoc;
+  AttributeNode attr;
+  DataNode composite;
   OrthConn *orth;
   Object *obj;
   int i;
@@ -605,23 +735,48 @@ association_load(int fd, int version)
   obj->type = &association_type;
   obj->ops = &association_ops;
 
-  orthconn_load(orth, fd);
+  orthconn_load(orth, obj_node);
+
+  assoc->name = NULL;
+  attr = object_find_attribute(obj_node, "name");
+  if (attr != NULL)
+    assoc->name = data_string(attribute_first_data(attr));
   
-  assoc->name = read_string(fd);
   assoc->text_width = 0.0;
   if (assoc->name != NULL) {
     assoc->text_width =
       font_string_width(assoc->name, assoc_font, ASSOCIATION_FONTHEIGHT);
   }
 
-  assoc->direction = read_int32(fd);
+  assoc->direction = ASSOC_NODIR;
+  attr = object_find_attribute(obj_node, "direction");
+  if (attr != NULL)
+    assoc->direction = data_enum(attribute_first_data(attr));
 
+  attr = object_find_attribute(obj_node, "ends");
+  composite = attribute_first_data(attr);
   for (i=0;i<2;i++) {
-    assoc->end[i].role = read_string(fd);
-    assoc->end[i].multiplicity = read_string(fd);
-    assoc->end[i].arrow = read_int32(fd);
-    assoc->end[i].aggregate = read_int32(fd);
+
+    assoc->end[i].role = NULL;
+    attr = composite_find_attribute(composite, "role");
+    if (attr != NULL)
+      assoc->end[i].role = data_string(attribute_first_data(attr));
     
+    assoc->end[i].multiplicity = NULL;
+    attr = composite_find_attribute(composite, "multiplicity");
+    if (attr != NULL)
+      assoc->end[i].multiplicity = data_string(attribute_first_data(attr));
+    
+    assoc->end[i].arrow = FALSE;
+    attr = composite_find_attribute(composite, "arrow");
+    if (attr != NULL)
+      assoc->end[i].arrow = data_boolean(attribute_first_data(attr));
+
+    assoc->end[i].aggregate = AGGREGATE_NONE;
+    attr = composite_find_attribute(composite, "aggregate");
+    if (attr != NULL)
+      assoc->end[i].aggregate = data_enum(attribute_first_data(attr));
+
     assoc->end[i].text_width = 0.0;
     if (assoc->end[i].role != NULL) {
       assoc->end[i].text_width = 
@@ -634,6 +789,7 @@ association_load(int fd, int version)
 	    font_string_width(assoc->end[i].multiplicity,
 			      assoc_font, ASSOCIATION_FONTHEIGHT) );
     }
+    composite = data_next(composite);
   }
 
   assoc->properties_dialog = NULL;
@@ -643,25 +799,21 @@ association_load(int fd, int version)
   return (Object *)assoc;
 }
 
-static void
-apply_callback(GtkWidget *widget, gpointer data)
+static ObjectChange *
+association_apply_properties(Association *assoc)
 {
-  Association *assoc;
   AssociationPropertiesDialog *prop_dialog;
   char *str;
   GtkWidget *menuitem;
   int i;
+  ObjectState *old_state;
   
-  assoc = (Association *)data;
   prop_dialog = assoc->properties_dialog;
 
-  (prop_dialog->changed_callback)((Object *)assoc,
-				  prop_dialog->changed_callback_data,
-				  BEFORE_CHANGE);
-
+  old_state = (ObjectState *)association_get_state(assoc);
+  
   /* Read from dialog and put in object: */
-  if (assoc->name != NULL)
-    g_free(assoc->name);
+  g_free(assoc->name);
   str = gtk_entry_get_text(prop_dialog->name);
   if (strlen(str) != 0)
     assoc->name = strdup(str);
@@ -682,8 +834,7 @@ apply_callback(GtkWidget *widget, gpointer data)
   for (i=0;i<2;i++) {
     AssociationEnd *end = &assoc->end[i];
     /* Role: */
-    if (end->role != NULL)
-      g_free(end->role);
+    g_free(end->role);
     str = gtk_entry_get_text(prop_dialog->end[i].role);
     if (strlen(str) != 0)
       end->role = strdup(str);
@@ -691,8 +842,7 @@ apply_callback(GtkWidget *widget, gpointer data)
       end->role = NULL;
 
     /* Multiplicity: */
-    if (end->multiplicity != NULL)
-      g_free(end->multiplicity);
+    g_free(end->multiplicity);
     str = gtk_entry_get_text(prop_dialog->end[i].multiplicity);
     if (strlen(str) != 0)
       end->multiplicity = strdup(str);
@@ -721,13 +871,10 @@ apply_callback(GtkWidget *widget, gpointer data)
 
   }
 
-    
-  
   association_update_data(assoc);
-  
-  (prop_dialog->changed_callback)((Object *)assoc,
-				  prop_dialog->changed_callback_data,
-				  AFTER_CHANGE);
+  return new_object_state_change((Object *)assoc, old_state, 
+				 (GetStateFunc)association_get_state,
+				 (SetStateFunc)association_set_state);
 }
 
 static void
@@ -757,11 +904,11 @@ fill_in_dialog(Association *assoc)
     else 
       gtk_entry_set_text(prop_dialog->end[i].multiplicity, "");
 
-    gtk_toggle_button_set_state(prop_dialog->end[i].draw_arrow,
+    gtk_toggle_button_set_active(prop_dialog->end[i].draw_arrow,
 				assoc->end[i].arrow);
-    gtk_toggle_button_set_state(prop_dialog->end[i].aggregate,
+    gtk_toggle_button_set_active(prop_dialog->end[i].aggregate,
 				assoc->end[i].aggregate == AGGREGATE_NORMAL);
-    gtk_toggle_button_set_state(prop_dialog->end[i].composition,
+    gtk_toggle_button_set_active(prop_dialog->end[i].composition,
 				assoc->end[i].aggregate == AGGREGATE_COMPOSITION);
   }
 }
@@ -780,23 +927,20 @@ mutex_aggregate_callback(GtkWidget *widget,
 
   for (i=0;i<2;i++) {
     if (prop_dialog->end[i].aggregate != button) {
-      gtk_toggle_button_set_state(prop_dialog->end[i].aggregate, 0);
+      gtk_toggle_button_set_active(prop_dialog->end[i].aggregate, 0);
     }
     if (prop_dialog->end[i].composition != button) {
-      gtk_toggle_button_set_state(prop_dialog->end[i].composition, 0);
+      gtk_toggle_button_set_active(prop_dialog->end[i].composition, 0);
     }
   }
 }
 
-static void
-association_show_properties(Association *assoc,
-			   ObjectChangedFunc *changed_callback,
-			   void *changed_callback_data)
+static GtkWidget *
+association_get_properties(Association *assoc)
 {
   AssociationPropertiesDialog *prop_dialog;
   GtkWidget *dialog;
   GtkWidget *frame;
-  GtkWidget *button;
   GtkWidget *entry;
   GtkWidget *hbox;
   GtkWidget *split_hbox;
@@ -815,34 +959,24 @@ association_show_properties(Association *assoc,
     prop_dialog = g_new(AssociationPropertiesDialog, 1);
     assoc->properties_dialog = prop_dialog;
 
-    prop_dialog->changed_callback = changed_callback;
-    prop_dialog->changed_callback_data = changed_callback_data;
-    
-    dialog = gtk_dialog_new();
+    dialog = gtk_vbox_new(FALSE, 0);
     prop_dialog->dialog = dialog;
     
-    gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
-			GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
-    
-    gtk_window_set_title (GTK_WINDOW (dialog), "Association properties");
-    gtk_container_border_width (GTK_CONTAINER (dialog), 5);
-
     /* Name entry: */
     hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new("Name:");
+    label = gtk_label_new(_("Name:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     entry = gtk_entry_new();
     prop_dialog->name = GTK_ENTRY(entry);
     gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
     gtk_widget_show (label);
     gtk_widget_show (entry);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			hbox, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (dialog), hbox, TRUE, TRUE, 0);
     gtk_widget_show(hbox);
 
     /* Direction entry: */
     hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new("Direction:");
+    label = gtk_label_new(_("Direction:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     
     omenu = gtk_option_menu_new ();
@@ -852,21 +986,21 @@ association_show_properties(Association *assoc,
     submenu = NULL;
     group = NULL;
     
-    menuitem = gtk_radio_menu_item_new_with_label (group, "None");
+    menuitem = gtk_radio_menu_item_new_with_label (group, _("None"));
     gtk_object_set_user_data(GTK_OBJECT(menuitem),
 			     GINT_TO_POINTER(ASSOC_NODIR));
     group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
     gtk_menu_append (GTK_MENU (menu), menuitem);
     gtk_widget_show (menuitem);
     
-    menuitem = gtk_radio_menu_item_new_with_label (group, "Right");
+    menuitem = gtk_radio_menu_item_new_with_label (group, _("Right"));
     gtk_object_set_user_data(GTK_OBJECT(menuitem),
 			     GINT_TO_POINTER(ASSOC_RIGHT));
     group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
     gtk_menu_append (GTK_MENU (menu), menuitem);
     gtk_widget_show (menuitem);
     
-    menuitem = gtk_radio_menu_item_new_with_label (group, "Left");
+    menuitem = gtk_radio_menu_item_new_with_label (group, _("Left"));
     gtk_object_set_user_data(GTK_OBJECT(menuitem),
 			     GINT_TO_POINTER(ASSOC_LEFT));
     group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
@@ -878,13 +1012,11 @@ association_show_properties(Association *assoc,
     
     gtk_widget_show (label);
     gtk_widget_show (omenu);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			hbox, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (dialog), hbox, TRUE, TRUE, 0);
     gtk_widget_show(hbox);
     
     split_hbox = gtk_hbox_new(TRUE, 5);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			split_hbox, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (dialog), split_hbox, TRUE, TRUE, 0);
     gtk_widget_show(split_hbox);
 
     group = NULL; /* For the radio-buttons */
@@ -892,23 +1024,23 @@ association_show_properties(Association *assoc,
     for (i=0;i<2;i++) {
       char *str;
       if (i==0)
-	str = "Side A";
+	str = _("Side A");
       else
-	str = "Side B";
+	str = _("Side B");
       frame = gtk_frame_new(str);
       
       vbox = gtk_vbox_new(FALSE, 5);
       /* End 'i' into vbox: */
       if (i==0)
-	label = gtk_label_new("Side A");
+	label = gtk_label_new(_("Side A"));
       else
-	label = gtk_label_new("Side B");
+	label = gtk_label_new(_("Side B"));
       
       gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
       
       /* Role entry: */
       hbox = gtk_hbox_new(FALSE, 5);
-      label = gtk_label_new("Role:");
+      label = gtk_label_new(_("Role:"));
       gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
       entry = gtk_entry_new();
       prop_dialog->end[i].role = GTK_ENTRY(entry);
@@ -920,7 +1052,7 @@ association_show_properties(Association *assoc,
 
       /* Multiplicity entry: */
       hbox = gtk_hbox_new(FALSE, 5);
-      label = gtk_label_new("Multiplicity:");
+      label = gtk_label_new(_("Multiplicity:"));
       gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
       entry = gtk_entry_new();
       prop_dialog->end[i].multiplicity = GTK_ENTRY(entry);
@@ -931,13 +1063,13 @@ association_show_properties(Association *assoc,
       gtk_widget_show(hbox);
 
       /* Show arrow: */
-      checkbox = gtk_check_button_new_with_label("Show arrow");
+      checkbox = gtk_check_button_new_with_label(_("Show arrow"));
       prop_dialog->end[i].draw_arrow = GTK_TOGGLE_BUTTON( checkbox );
       gtk_widget_show(checkbox);
       gtk_box_pack_start (GTK_BOX (vbox), checkbox, TRUE, TRUE, 0);
 
       /* Aggregate */
-      checkbox = gtk_check_button_new_with_label("Aggregate");
+      checkbox = gtk_check_button_new_with_label(_("Aggregate"));
       prop_dialog->end[i].aggregate = GTK_TOGGLE_BUTTON( checkbox );
       gtk_signal_connect(GTK_OBJECT(checkbox), "toggled",
 			 (GtkSignalFunc) mutex_aggregate_callback, prop_dialog);
@@ -945,39 +1077,25 @@ association_show_properties(Association *assoc,
       gtk_box_pack_start (GTK_BOX (vbox), checkbox, TRUE, TRUE, 0);
 
       /* Composition */
-      checkbox = gtk_check_button_new_with_label("Composition");
+      checkbox = gtk_check_button_new_with_label(_("Composition"));
       prop_dialog->end[i].composition = GTK_TOGGLE_BUTTON( checkbox );
       gtk_signal_connect(GTK_OBJECT(checkbox), "toggled",
 			 (GtkSignalFunc) mutex_aggregate_callback, prop_dialog);
       gtk_widget_show(checkbox);
       gtk_box_pack_start (GTK_BOX (vbox), checkbox, TRUE, TRUE, 0);
 	
-      gtk_container_border_width(GTK_CONTAINER(vbox), 5);
+      gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
       gtk_container_add(GTK_CONTAINER(frame), vbox);
       gtk_box_pack_start (GTK_BOX (split_hbox), frame, TRUE, TRUE, 0);
       gtk_widget_show(vbox);
       gtk_widget_show(frame);
     }
 
-    button = gtk_button_new_with_label ("Apply");
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
-			button, TRUE, TRUE, 0);
-    gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC(apply_callback),
-			assoc);
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC(gtk_widget_hide),
-			GTK_OBJECT(dialog));
-    gtk_widget_grab_default (button);
-    gtk_widget_show (button);
-  } else {
-    assoc->properties_dialog->changed_callback = changed_callback;
-    assoc->properties_dialog->changed_callback_data = changed_callback_data;
-
   }
   fill_in_dialog(assoc);
   gtk_widget_show (assoc->properties_dialog->dialog);
+
+  return assoc->properties_dialog->dialog;
 }
 
 
