@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,12 +20,12 @@
 #include <math.h>
 #include <string.h>
 
+#include "config.h"
+#include "intl.h"
 #include "object.h"
 #include "orth_conn.h"
 #include "render.h"
 #include "attributes.h"
-#include "files.h"
-#include "sheet.h"
 #include "arrows.h"
 
 #include "uml.h"
@@ -33,8 +33,17 @@
 #include "pixmaps/realizes.xpm"
 
 typedef struct _Realizes Realizes;
+typedef struct _RealizesState RealizesState;
 typedef struct _RealizesPropertiesDialog RealizesPropertiesDialog;
 
+struct _RealizesState {
+  ObjectState obj_state;
+
+  char *name;
+  char *stereotype; 
+
+};
+
 struct _Realizes {
   OrthConn orth;
 
@@ -53,9 +62,6 @@ struct _RealizesPropertiesDialog {
   
   GtkEntry *name;
   GtkEntry *stereotype;
-  
-  ObjectChangedFunc *changed_callback;
-  void *changed_callback_data;
 };
 
 #define REALIZES_WIDTH 0.1
@@ -69,7 +75,7 @@ static real realizes_distance_from(Realizes *realize, Point *point);
 static void realizes_select(Realizes *realize, Point *clicked_point,
 			      Renderer *interactive_renderer);
 static void realizes_move_handle(Realizes *realize, Handle *handle,
-				   Point *to, HandleMoveReason reason);
+				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
 static void realizes_move(Realizes *realize, Point *to);
 static void realizes_draw(Realizes *realize, Renderer *renderer);
 static Object *realizes_create(Point *startpoint,
@@ -78,12 +84,19 @@ static Object *realizes_create(Point *startpoint,
 				 Handle **handle2);
 static void realizes_destroy(Realizes *realize);
 static Object *realizes_copy(Realizes *realize);
-static void realizes_show_properties(Realizes *realize,
-				       ObjectChangedFunc *changed_callback,
-				       void *changed_callback_data);
+static GtkWidget *realizes_get_properties(Realizes *realize);
+static ObjectChange *realizes_apply_properties(Realizes *realize);
+static DiaMenu *realizes_get_object_menu(Realizes *realize,
+					 Point *clickedpoint);
 
-static void realizes_save(Realizes *realize, int fd);
-static Object *realizes_load(int fd, int version);
+static RealizesState *realizes_get_state(Realizes *realize);
+static void realizes_set_state(Realizes *realize,
+			       RealizesState *state);
+
+static void realizes_save(Realizes *realize, ObjectNode obj_node,
+			  const char *filename);
+static Object *realizes_load(ObjectNode obj_node, int version,
+			     const char *filename);
 
 static void realizes_update_data(Realizes *realize);
 
@@ -103,26 +116,17 @@ ObjectType realizes_type =
   &realizes_type_ops       /* ops */
 };
 
-SheetObject realizes_sheetobj =
-{
-  &realizes_type,             /* type */
-  "Realizes, implements a specific interface.",
-                              /* description */
-  (char **) realizes_xpm,     /* pixmap */
-
-  NULL                        /* user_data */
-};
-
 static ObjectOps realizes_ops = {
-  (DestroyFunc)        realizes_destroy,
-  (DrawFunc)           realizes_draw,
-  (DistanceFunc)       realizes_distance_from,
-  (SelectFunc)         realizes_select,
-  (CopyFunc)           realizes_copy,
-  (MoveFunc)           realizes_move,
-  (MoveHandleFunc)     realizes_move_handle,
-  (ShowPropertiesFunc) realizes_show_properties,
-  (IsEmptyFunc)        object_return_false
+  (DestroyFunc)         realizes_destroy,
+  (DrawFunc)            realizes_draw,
+  (DistanceFunc)        realizes_distance_from,
+  (SelectFunc)          realizes_select,
+  (CopyFunc)            realizes_copy,
+  (MoveFunc)            realizes_move,
+  (MoveHandleFunc)      realizes_move_handle,
+  (GetPropertiesFunc)   realizes_get_properties,
+  (ApplyPropertiesFunc) realizes_apply_properties,
+  (ObjectMenuFunc)      realizes_get_object_menu
 };
 
 static real
@@ -141,7 +145,7 @@ realizes_select(Realizes *realize, Point *clicked_point,
 
 static void
 realizes_move_handle(Realizes *realize, Handle *handle,
-		       Point *to, HandleMoveReason reason)
+		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(realize!=NULL);
   assert(handle!=NULL);
@@ -221,8 +225,6 @@ realizes_update_data(Realizes *realize)
   obj->bounding_box.bottom += REALIZES_WIDTH/2.0 + REALIZES_TRIANGLESIZE;
   obj->bounding_box.right += REALIZES_WIDTH/2.0 + REALIZES_TRIANGLESIZE;
   
-  obj->position = orth->points[0];
-
   /* Calc text pos: */
   num_segm = realize->orth.numpoints - 1;
   points = realize->orth.points;
@@ -258,6 +260,49 @@ realizes_update_data(Realizes *realize)
   rectangle_union(&obj->bounding_box, &rect);
 }
 
+static ObjectChange *
+realizes_add_segment_callback(Object *obj, Point *clicked, gpointer data)
+{
+  ObjectChange *change;
+  change = orthconn_add_segment((OrthConn *)obj, clicked);
+  realizes_update_data((Realizes *)obj);
+  return change;
+}
+
+static ObjectChange *
+realizes_delete_segment_callback(Object *obj, Point *clicked, gpointer data)
+{
+  ObjectChange *change;
+  change = orthconn_delete_segment((OrthConn *)obj, clicked);
+  realizes_update_data((Realizes *)obj);
+  return change;
+}
+
+
+static DiaMenuItem object_menu_items[] = {
+  { N_("Add segment"), realizes_add_segment_callback, NULL, 1 },
+  { N_("Delete segment"), realizes_delete_segment_callback, NULL, 1 },
+};
+
+static DiaMenu object_menu = {
+  "Realizes",
+  sizeof(object_menu_items)/sizeof(DiaMenuItem),
+  object_menu_items,
+  NULL
+};
+
+static DiaMenu *
+realizes_get_object_menu(Realizes *realize, Point *clickedpoint)
+{
+  OrthConn *orth;
+
+  orth = &realize->orth;
+  /* Set entries sensitive/selected etc here */
+  object_menu_items[0].active = orthconn_can_add_segment(orth, clickedpoint);
+  object_menu_items[1].active = orthconn_can_delete_segment(orth, clickedpoint);
+  return &object_menu;
+}
+
 static Object *
 realizes_create(Point *startpoint,
 	       void *user_data,
@@ -282,18 +327,15 @@ realizes_create(Point *startpoint,
 
   orthconn_init(orth, startpoint);
   
-  realizes_update_data(realize);
-
   realize->name = NULL;
   realize->stereotype = NULL;
-
   realize->text_width = 0;
-  
   realize->properties_dialog = NULL;
-  
-  *handle1 = &orth->endpoint_handles[0];
-  *handle2 = &orth->endpoint_handles[1];
 
+  realizes_update_data(realize);
+  
+  *handle1 = orth->handles[0];
+  *handle2 = orth->handles[orth->numpoints-2];
   return (Object *)realize;
 }
 
@@ -331,20 +373,66 @@ realizes_copy(Realizes *realize)
   return (Object *)newrealize;
 }
 
+static void
+realizes_state_free(ObjectState *ostate)
+{
+  RealizesState *state = (RealizesState *)ostate;
+  g_free(state->name);
+  g_free(state->stereotype);
+}
+
+static RealizesState *
+realizes_get_state(Realizes *realize)
+{
+  RealizesState *state = g_new(RealizesState, 1);
+
+  state->obj_state.free = realizes_state_free;
+
+  state->name = g_strdup(realize->name);
+  state->stereotype = g_strdup(realize->stereotype);
+  
+  return state;
+}
 
 static void
-realizes_save(Realizes *realize, int fd)
+realizes_set_state(Realizes *realize, RealizesState *state)
 {
-  orthconn_save(&realize->orth, fd);
+  g_free(realize->name);
+  g_free(realize->stereotype);
+  realize->name = state->name;
+  realize->stereotype = state->stereotype;
+  
+  realize->text_width = 0.0;
+  if (realize->name != NULL) {
+    realize->text_width =
+      font_string_width(realize->name, realize_font, REALIZES_FONTHEIGHT);
+  }
+  if (realize->stereotype != NULL) {
+    realize->text_width = MAX(realize->text_width,
+			  font_string_width(realize->stereotype, realize_font, REALIZES_FONTHEIGHT));
+  }
+  
+  g_free(state);
+  
+  realizes_update_data(realize);
+}
 
-  write_string(fd, realize->name);
-  write_string(fd, realize->stereotype);
+static void
+realizes_save(Realizes *realize, ObjectNode obj_node, const char *filename)
+{
+  orthconn_save(&realize->orth, obj_node);
+
+  data_add_string(new_attribute(obj_node, "name"),
+		  realize->name);
+  data_add_string(new_attribute(obj_node, "stereotype"),
+		  realize->stereotype);
 }
 
 static Object *
-realizes_load(int fd, int version)
+realizes_load(ObjectNode obj_node, int version, const char *filename)
 {
   Realizes *realize;
+  AttributeNode attr;
   OrthConn *orth;
   Object *obj;
 
@@ -360,10 +448,17 @@ realizes_load(int fd, int version)
   obj->type = &realizes_type;
   obj->ops = &realizes_ops;
 
-  orthconn_load(orth, fd);
+  orthconn_load(orth, obj_node);
+
+  realize->name = NULL;
+  attr = object_find_attribute(obj_node, "name");
+  if (attr != NULL)
+    realize->name = data_string(attribute_first_data(attr));
   
-  realize->name = read_string(fd);
-  realize->stereotype = read_string(fd);
+  realize->stereotype = NULL;
+  attr = object_find_attribute(obj_node, "stereotype");
+  if (attr != NULL)
+    realize->stereotype = data_string(attribute_first_data(attr));
   
   realize->text_width = 0.0;
 
@@ -383,19 +478,16 @@ realizes_load(int fd, int version)
   return (Object *)realize;
 }
 
-static void
-apply_callback(GtkWidget *widget, gpointer data)
+static ObjectChange *
+realizes_apply_properties(Realizes *realize)
 {
-  Realizes *realize;
   RealizesPropertiesDialog *prop_dialog;
+  ObjectState *old_state;
   char *str;
 
-  realize = (Realizes *)data;
   prop_dialog = realize->properties_dialog;
 
-  (prop_dialog->changed_callback)((Object *)realize,
-				  prop_dialog->changed_callback_data,
-				  BEFORE_CHANGE);
+  old_state = (ObjectState *)realizes_get_state(realize);
 
   /* Read from dialog and put in object: */
   if (realize->name != NULL)
@@ -434,10 +526,9 @@ apply_callback(GtkWidget *widget, gpointer data)
   }
   
   realizes_update_data(realize);
-  
-  (prop_dialog->changed_callback)((Object *)realize,
-				  prop_dialog->changed_callback_data,
-				  AFTER_CHANGE);
+  return new_object_state_change((Object *)realize, old_state, 
+				 (GetStateFunc)realizes_get_state,
+				 (SetStateFunc)realizes_set_state);
 }
 
 static void
@@ -465,14 +556,11 @@ fill_in_dialog(Realizes *realize)
   }
 }
 
-static void
-realizes_show_properties(Realizes *realize,
-			   ObjectChangedFunc *changed_callback,
-			   void *changed_callback_data)
+static GtkWidget *
+realizes_get_properties(Realizes *realize)
 {
   RealizesPropertiesDialog *prop_dialog;
   GtkWidget *dialog;
-  GtkWidget *button;
   GtkWidget *entry;
   GtkWidget *hbox;
   GtkWidget *label;
@@ -481,65 +569,38 @@ realizes_show_properties(Realizes *realize,
 
     prop_dialog = g_new(RealizesPropertiesDialog, 1);
     realize->properties_dialog = prop_dialog;
-
-    prop_dialog->changed_callback = changed_callback;
-    prop_dialog->changed_callback_data = changed_callback_data;
     
-    dialog = gtk_dialog_new();
+    dialog = gtk_vbox_new(FALSE, 0);
     prop_dialog->dialog = dialog;
     
-    gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
-			GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
-    
-    gtk_window_set_title (GTK_WINDOW (dialog), "Realizes properties");
-    gtk_container_border_width (GTK_CONTAINER (dialog), 5);
-
     hbox = gtk_hbox_new(FALSE, 5);
 
-    label = gtk_label_new("Name:");
+    label = gtk_label_new(_("Name:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     entry = gtk_entry_new();
     prop_dialog->name = GTK_ENTRY(entry);
     gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
     gtk_widget_show (label);
     gtk_widget_show (entry);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			hbox, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (dialog), hbox, TRUE, TRUE, 0);
     gtk_widget_show(hbox);
 
     hbox = gtk_hbox_new(FALSE, 5);
-    gtk_container_border_width (GTK_CONTAINER (hbox), 5);
-    label = gtk_label_new("Stereotype:");
+    gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
+    label = gtk_label_new(_("Stereotype:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     entry = gtk_entry_new();
     prop_dialog->stereotype = GTK_ENTRY(entry);
     gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
     gtk_widget_show (label);
     gtk_widget_show (entry);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			hbox, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (dialog), hbox, TRUE, TRUE, 0);
     gtk_widget_show(hbox);
-    
-
-    button = gtk_button_new_with_label ("Apply");
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
-			button, TRUE, TRUE, 0);
-    gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC(apply_callback),
-			realize);
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC(gtk_widget_hide),
-			GTK_OBJECT(dialog));
-    gtk_widget_grab_default (button);
-    gtk_widget_show (button);
-  } else {
-    realize->properties_dialog->changed_callback = changed_callback;
-    realize->properties_dialog->changed_callback_data = changed_callback_data;
-
   }
   fill_in_dialog(realize);
   gtk_widget_show (realize->properties_dialog->dialog);
+
+  return realize->properties_dialog->dialog;
 }
 
 
