@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,12 +20,12 @@
 #include <math.h>
 #include <string.h>
 
+#include "config.h"
+#include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "render.h"
 #include "attributes.h"
-#include "files.h"
-#include "sheet.h"
 #include "text.h"
 
 #include "pixmaps/smallpackage.xpm"
@@ -51,7 +51,7 @@ static real smallpackage_distance_from(SmallPackage *pkg, Point *point);
 static void smallpackage_select(SmallPackage *pkg, Point *clicked_point,
 				Renderer *interactive_renderer);
 static void smallpackage_move_handle(SmallPackage *pkg, Handle *handle,
-				     Point *to, HandleMoveReason reason);
+				     Point *to, HandleMoveReason reason, ModifierKeys modifiers);
 static void smallpackage_move(SmallPackage *pkg, Point *to);
 static void smallpackage_draw(SmallPackage *pkg, Renderer *renderer);
 static Object *smallpackage_create(Point *startpoint,
@@ -61,8 +61,10 @@ static Object *smallpackage_create(Point *startpoint,
 static void smallpackage_destroy(SmallPackage *pkg);
 static Object *smallpackage_copy(SmallPackage *pkg);
 
-static void smallpackage_save(SmallPackage *pkg, int fd);
-static Object *smallpackage_load(int fd, int version);
+static void smallpackage_save(SmallPackage *pkg, ObjectNode obj_node,
+			      const char *filename);
+static Object *smallpackage_load(ObjectNode obj_node, int version,
+				 const char *filename);
 
 static void smallpackage_update_data(SmallPackage *pkg);
 
@@ -82,25 +84,17 @@ ObjectType smallpackage_type =
   &smallpackage_type_ops       /* ops */
 };
 
-SheetObject smallpackage_sheetobj =
-{
-  &smallpackage_type,             /* type */
-  "Create a (small) package",           /* description */
-  (char **) smallpackage_xpm,     /* pixmap */
-
-  NULL                        /* user_data */
-};
-
 static ObjectOps smallpackage_ops = {
-  (DestroyFunc)        smallpackage_destroy,
-  (DrawFunc)           smallpackage_draw,
-  (DistanceFunc)       smallpackage_distance_from,
-  (SelectFunc)         smallpackage_select,
-  (CopyFunc)           smallpackage_copy,
-  (MoveFunc)           smallpackage_move,
-  (MoveHandleFunc)     smallpackage_move_handle,
-  (ShowPropertiesFunc) object_show_properties_none,
-  (IsEmptyFunc)        object_return_false
+  (DestroyFunc)         smallpackage_destroy,
+  (DrawFunc)            smallpackage_draw,
+  (DistanceFunc)        smallpackage_distance_from,
+  (SelectFunc)          smallpackage_select,
+  (CopyFunc)            smallpackage_copy,
+  (MoveFunc)            smallpackage_move,
+  (MoveHandleFunc)      smallpackage_move_handle,
+  (GetPropertiesFunc)   object_return_null,
+  (ApplyPropertiesFunc) object_return_void,
+  (ObjectMenuFunc)      NULL
 };
 
 static real
@@ -121,7 +115,7 @@ smallpackage_select(SmallPackage *pkg, Point *clicked_point,
 
 static void
 smallpackage_move_handle(SmallPackage *pkg, Handle *handle,
-			 Point *to, HandleMoveReason reason)
+			 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(pkg!=NULL);
   assert(handle!=NULL);
@@ -321,17 +315,20 @@ smallpackage_copy(SmallPackage *pkg)
 
 
 static void
-smallpackage_save(SmallPackage *pkg, int fd)
+smallpackage_save(SmallPackage *pkg, ObjectNode obj_node,
+		  const char *filename)
 {
-  element_save(&pkg->element, fd);
+  element_save(&pkg->element, obj_node);
 
-  write_text(fd, pkg->text);
+  data_add_text(new_attribute(obj_node, "text"),
+		pkg->text);
 }
 
 static Object *
-smallpackage_load(int fd, int version)
+smallpackage_load(ObjectNode obj_node, int version, const char *filename)
 {
   SmallPackage *pkg;
+  AttributeNode attr;
   Element *elem;
   Object *obj;
   int i;
@@ -343,9 +340,12 @@ smallpackage_load(int fd, int version)
   obj->type = &smallpackage_type;
   obj->ops = &smallpackage_ops;
 
-  element_load(elem, fd);
+  element_load(elem, obj_node);
   
-  pkg->text = read_text(fd);
+  pkg->text = NULL;
+  attr = object_find_attribute(obj_node, "text");
+  if (attr != NULL)
+    pkg->text = data_text(attribute_first_data(attr));
   
   element_init(elem, 8, 8);
 
