@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,13 +23,26 @@ extern ObjectType *_box_type;
 extern ObjectType *_ellipse_type;
 extern ObjectType *_line_type;
 extern ObjectType *_zigzagline_type;
+extern ObjectType *_polyline_type;
+extern ObjectType *_bezierline_type;
 extern ObjectType *_textobj_type;
+extern ObjectType *_image_type;
+
+int get_version(void) {
+  return 0;
+}
 
 void register_objects(void) {
   object_register_type(_arc_type);
   object_register_type(_box_type);
   object_register_type(_ellipse_type);
   object_register_type(_line_type);
+  object_register_type(_polyline_type);
   object_register_type(_zigzagline_type);
+  object_register_type(_bezierline_type);
   object_register_type(_textobj_type);
+  object_register_type(_image_type);
+}
+
+void register_sheets(void) {
 }
